// @(#)root/gui:$Id$
// Author: Fons Rademakers   28/07/97

/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifndef ROOT_HelpText
#define ROOT_HelpText

#include "RtypesCore.h"

R__EXTERN const char gHelpAbout[];
R__EXTERN const char gHelpBrowser[];
R__EXTERN const char gHelpBrowserLite[];
R__EXTERN const char gHelpGLViewer[];
R__EXTERN const char gHelpPostscript[];
R__EXTERN const char gHelpButtons[];
R__EXTERN const char gHelpGraphicsEditor[];
R__EXTERN const char gHelpPullDownMenus[];
R__EXTERN const char gHelpCanvas[];
R__EXTERN const char gHelpObjects[];
R__EXTERN const char gHelpTextEditor[];
R__EXTERN const char gHelpRemote[];

#endif
