/*
 * Project: RooFit
 * Author:
 *   Will Buttinger, RAL 2022
 *
 * Copyright (c) 2022, CERN
 *
 * Redistribution and use in source and binary forms,
 * with or without modification, are permitted according to the terms
 * listed in LICENSE (http://roofit.sourceforge.net/license.txt)
 */


#ifndef RooFit_RooBrowser_h
#define RooFit_RooBrowser_h

#include "RooFit/xRooFit/xRooBrowser.h"

namespace ROOT::Experimental {
using RooBrowser = XROOFIT_NAMESPACE::xRooBrowser;
}

#endif
