// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

#ifndef ROOT_Math_GenVector_RotationZfwd
#define ROOT_Math_GenVector_RotationZfwd  1


namespace ROOT {

  namespace Math {

  /**
      Class describing a rotation about the Z axis
   */

class RotationZ;

}  // namespace Math
}  // namespace ROOT

#endif  // ROOT_Math_GenVector_RotationZfwd
