// @(#)root/mathcore:$Id$
// Authors: L. Moneta, A. Zsenei   08/2005

 /**********************************************************************
  *                                                                    *
  * Copyright (c) 2004 ROOT Foundation,  CERN/PH-SFT                   *
  *                                                                    *
  *                                                                    *
  **********************************************************************/

#ifndef ROOT_Math_DistFuncMathCore
#define ROOT_Math_DistFuncMathCore



// pdf functions from Mathcore
#include "Math/PdfFuncMathCore.h"

// cdf from mathcore
#include "Math/ProbFuncMathCore.h"

// quantile functions from mathcore
#include "Math/QuantFuncMathCore.h"


#endif  // ROOT_Math_DistFunc
