// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

#ifndef ROOT_Math_GenVector_PositionVector2Dfwd
#define ROOT_Math_GenVector_PositionVector2Dfwd  1


namespace ROOT {

  namespace Math {

     /**
         Class describing a 2D Position vector
     */

     template <class CoordSystem, class Tag>
     class PositionVector2D;

  }
}

#endif
