#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TNetXNGFile;
#pragma link C++ class TNetXNGFileStager;
#pragma link C++ class TNetXNGSystem;

#endif
