/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.source.less";
@import "../../../../../sap/ui/core/themes/sap_horizon/base.less";
@import "../../../../../sap/ui/core/themes/sap_horizon/global.less";

@import "BlockLayout.less";
@import "FormLayout.less";
@import "ColumnLayout.less";
@import "GridLayout.less";
@import "ResponsiveGridLayout.less";
@import "ResponsiveLayout.less";
@import "ResponsiveSplitter.less";
@import "Splitter.less";