ace.define("ace/mode/plain_text",["require","exports","module","ace/lib/oop","ace/mode/text","ace/mode/text_highlight_rules","ace/mode/behaviour"],function(e,t,i){"use strict";var o=e("../lib/oop");var r=e("./text").Mode;var a=e("./text_highlight_rules").TextHighlightRules;var n=e("./behaviour").Behaviour;var u=function(){this.HighlightRules=a;this.$behaviour=new n};o.inherits(u,r);(function(){this.type="text";this.getNextLineIndent=function(e,t,i){return""};this.$id="ace/mode/plain_text"}).call(u.prototype);t.Mode=u});(function(){ace.require(["ace/mode/plain_text"],function(e){if(typeof module=="object"&&typeof exports=="object"&&module){module.exports=e}})})();
//# sourceMappingURL=mode-plain_text.js.map