/* @(#)root/io:$Id$ */

/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TBufferIO;
#pragma link C++ class TBufferFile;
#pragma link C++ class TBufferText;
#pragma link C++ class TBufferJSON;
#pragma link C++ class TDirectoryFile-;
#pragma link C++ class TFile-;
#pragma link C++ class TFileCacheRead+;
#pragma link C++ class TFileCacheWrite+;
#pragma link C++ class TFileMerger+;
#pragma link C++ class TFree;
#pragma link C++ class TKey-;
#pragma link C++ class TKeyMapFile;
#pragma link C++ class TMapFile;
#pragma link C++ class TMapRec;
#pragma link C++ class TMemFile;
#pragma link C++ class TArchiveFile+;
#pragma link C++ class TArchiveMember+;
#pragma link C++ class TZIPFile+;
#pragma link C++ class TZIPMember+;
#pragma link C++ class TLockFile;
#pragma link C++ class TStreamerInfo-;
#pragma link C++ class TCollectionProxyFactory-;
#pragma link C++ class TEmulatedCollectionProxy-;
#pragma link C++ class TEmulatedMapProxy-;
#pragma link C++ class TGenCollectionProxy-;
#pragma link C++ class TGenCollectionProxy::Value-;
#pragma link C++ class TGenCollectionProxy::Method-;
#pragma link C++ class TCollectionStreamer-;
#pragma link C++ class TCollectionClassStreamer-;
#pragma link C++ class TCollectionMemberStreamer-;
#pragma link C++ class TFPBlock+;
#pragma link C++ class TFilePrefetch+;
#pragma link C++ namespace TStreamerInfoActions;
#pragma link C++ class TStreamerInfoActions::TConfiguredAction+;
#pragma link C++ class TStreamerInfoActions::TActionSequence+;
#pragma link C++ class TStreamerInfoActions::TConfiguration-;
#pragma link C++ class ROOT::Internal::RRawFile+;
#pragma link C++ class ROOT::TBufferMerger;
#pragma link C++ class ROOT::TBufferMergerFile;

#endif
