// @(#)root/recorder:$Id$

/*************************************************************************
 * Copyright (C) 1995-2008, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TRecEvent;
#pragma link C++ class TRecCmdEvent;
#pragma link C++ class TRecExtraEvent;
#pragma link C++ class TRecGuiEvent;
#pragma link C++ class TRecWinPair;
#pragma link C++ class TRecorder;
#pragma link C++ class TRecorderState;
#pragma link C++ class TRecorderReplaying;
#pragma link C++ class TRecorderRecording;
#pragma link C++ class TRecorderInactive;
#pragma link C++ class TRecorderPaused;
#pragma link C++ class TGRecorder;

#endif
