/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.source.less";
@import "../../../../../sap/ui/core/themes/sap_horizon/base.less";
@import "../../../../../sap/ui/core/themes/sap_horizon/global.less";

@_sap_ui_table_DeleteIcon: decline;
@_sap_ui_table_RowActionItemSize: @_sap_ui_table_BaseSizeCondensed;
@_sap_ui_table_RowActionItemSizeCozy: @_sap_ui_table_BaseSize;
@_sap_ui_table_RowActionItemSizeCondensed: calc(@_sap_ui_table_BaseSizeCondensed ~"-" 0.125rem);

@import "Table.less";
@import "Cell.less";
@import "RowSelection.less";
@import "RowAction.less";
@import "Scrolling.less";
@import "Grouping.less";