#ifndef ROOT_RConfigure
#define ROOT_RConfigure

/* Configurations file for linuxx8664gcc */

#define R__HAVE_CONFIG

#ifdef R__HAVE_CONFIG
#define ROOTPREFIX    "/usr"
#define ROOTBINDIR    "/usr/bin"
#define ROOTLIBDIR    "/usr/lib/root"
#define ROOTETCDIR    "/etc/root"
#define ROOTDATADIR   "/usr/share/root"
#define ROOTDOCDIR    "/usr/share/doc/root"
#define ROOTMACRODIR  "/usr/share/root/macros"
#define ROOTTUTDIR    "/usr/share/doc/root/tutorials"
#define ROOTSRCDIR    "/usr/share/root/src"
#define ROOTICONPATH  "/usr/share/root/icons"
#define TTFFONTDIR    "/usr/share/root/fonts"
#endif

#define EXTRAICONPATH ""

#define ROOT__cplusplus 202002L
#if defined(__cplusplus) && (__cplusplus != ROOT__cplusplus)
# if defined(_MSC_VER)
#  pragma message(__FILE__ ": Warning: The C++ standard in this build does not match ROOT configuration (202002L); this might cause unexpected issues. And please make sure you are using the -Zc:__cplusplus compilation flag")
# else
#  warning "The C++ standard in this build does not match ROOT configuration (202002L); this might cause unexpected issues"
# endif
#endif

#define R__HAS_SETRESUID   /**/
#define R__HAS_MATHMORE   /**/
#define R__HAS_PTHREAD    /**/
#define R__HAS_XFT    /**/
#define R__HAS_CLAD    /**/
#undef R__HAS_COCOA    /**/
#define R__HAS_VDT    /**/
#define R__HAS_STD_EXPERIMENTAL_SIMD    /**/
#undef R__EXPERIMENTAL_SIMD_PIN_AVX_ABI    /**/
#undef R__USE_CXXMODULES   /**/
#undef R__USE_LIBCXX    /**/
#define R__HAS_ATTRIBUTE_ALWAYS_INLINE /**/
#define R__HAS_ATTRIBUTE_NOINLINE /**/
#define R__USE_IMT   /**/
#undef R__COMPLETE_MEM_TERMINATION /**/
#undef R__HAS_CEFWEB  /**/
#define R__HAS_QT6WEB  /**/
#undef R__HAS_DAVIX  /**/
#define R__HAS_CURL  /**/
#define R__HAS_DATAFRAME /**/
#define R__HAS_ROOT7 /**/
#undef R__LESS_INCLUDES /**/
#define R__HARDWARE_INTERFERENCE_SIZE 64
 /*Determined at CMake configure to be stable across all TUs*/

#undef R__HAS_ZLIB_NG /**/

#undef R__HAS_TMVACPU /**/
#define R__HAS_TMVAGPU /**/
#define R__HAS_CUDNN /**/
#define R__HAS_PYMVA /**/
#undef R__HAS_RMVA /**/

#define R__HAS_URING /**/

#define R__HAS_GEOM /**/

#endif
