#ifndef roofit_roofitcore_RooFit_RooListProxy_h
#define roofit_roofitcore_RooFit_RooListProxy_h

#include <RooArgList.h>
#include <RooCollectionProxy.h>

using RooListProxy = RooCollectionProxy<RooArgList>;

#endif
