/// \file
/// \ingroup Tutorials
/// Example of `rootlogoff.C`.
/// The macro `rootlogoff.C` in the current working directory, is executed when
/// `root` finishes.
///
/// \macro_code
///
/// \author Rene Brun

{
   printf("\nTaking a break from ROOT? Hope to see you back!\n\n");
}
