/// \file ROOT/RNTupleUtil.hxx
/// \ingroup NTuple
/// \author Jakob Blomer <jblomer@cern.ch>
/// \date 2025-07-31

/*************************************************************************
 * Copyright (C) 1995-2025, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifndef ROOT_RNTupleUtil
#define ROOT_RNTupleUtil

#if !defined(R__NO_DEPRECATION) && !defined(__CLING__)
#warning "<ROOT/RNTupleUtil.hxx> is deprecated and will be removed in ROOT v6.40. Use <ROOT/RNTupleTypes.hxx>."
#endif

#include <ROOT/RNTupleTypes.hxx>
#include <ROOT/RNTupleUtils.hxx>

#endif
