// @(#)root/matrix:$Id$
// Authors: Fons Rademakers, Eddy Offermann   Nov 2003

/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifndef ROOT_TMatrixFUtils
#define ROOT_TMatrixFUtils

//////////////////////////////////////////////////////////////////////////
//                                                                      //
// Matrix utility classes.                                              //
//                                                                      //
//  Instantiation of                                                     //
//   TMatrixTRow_const       <Float_t>  TMatrixTRow       <Float_t>     //
//   TMatrixTColumn_const    <Float_t>  TMatrixTColumn    <Float_t>     //
//   TMatrixTDiag_const      <Float_t>  TMatrixTDiag      <Float_t>     //
//   TMatrixTFlat_const      <Float_t>  TMatrixTFlat      <Float_t>     //
//   TMatrixTSub_const       <Float_t>  TMatrixTSub       <Float_t>     //
//   TMatrixTSparseRow_const <Float_t>  TMatrixTSparseRow <Float_t>     //
//   TMatrixTSparseDiag_const<Float_t>  TMatrixTSparseDiag<Float_t>     //
//                                                                      //
//   TElementActionT   <Float_t>                                        //
//   TElementPosActionT<Float_t>                                        //
//                                                                      //
//////////////////////////////////////////////////////////////////////////

#include "TMatrixTUtils.h"
#include "TMatrixFUtilsfwd.h"

#endif
