#include "TMVA/ROperator_Transpose.hxx"
#include "TMVA/ROperator_Gemm.hxx"
#include "TMVA/ROperator_Relu.hxx"
#include "TMVA/ROperator_Tanh.hxx"
#include "TMVA/ROperator_LeakyRelu.hxx"
#include "TMVA/ROperator_Selu.hxx"
#include "TMVA/ROperator_Sigmoid.hxx"
#include "TMVA/ROperator_Conv.hxx"
#include "TMVA/ROperator_RNN.hxx"
#include "TMVA/ROperator_LSTM.hxx"
#include "TMVA/ROperator_BatchNormalization.hxx"
#include "TMVA/ROperator_Pool.hxx"
#include "TMVA/ROperator_BasicUnary.hxx"
#include "TMVA/ROperator_BasicBinary.hxx"
#include "TMVA/ROperator_Reshape.hxx"
#include "TMVA/ROperator_Slice.hxx"
#include "TMVA/ROperator_GRU.hxx"
#include "TMVA/ROperator_Identity.hxx"
#include "TMVA/ROperator_Softmax.hxx"
#include "TMVA/ROperator_BasicNary.hxx"
#include "TMVA/ROperator_Concat.hxx"
#include "TMVA/ROperator_Reduce.hxx"
#include "TMVA/ROperator_Cast.hxx"
#include "TMVA/ROperator_Shape.hxx"
#include "TMVA/ROperator_ConvTranspose.hxx"
#include "TMVA/ROperator_Custom.hxx"
#include "TMVA/ROperator_LayerNormalization.hxx"
#include "TMVA/ROperator_Expand.hxx"
#include "TMVA/ROperator_Gather.hxx"
#include "TMVA/ROperator_Swish.hxx"
#include "TMVA/ROperator_Erf.hxx"
#include "TMVA/ROperator_Elu.hxx"
#include "TMVA/ROperator_Comparision.hxx"
#include "TMVA/ROperator_EyeLike.hxx"
#include "TMVA/ROperator_Range.hxx"

