/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.less";
@import "../../../../../../Base/baseLib/sap_horizon_dark/base.less"; /* ORIGINAL IMPORT PATH: "../../../../../sap/ui/core/themes/sap_horizon_dark/base.less" */

@import "../../../../../sap/ui/core/themes/sap_horizon_dark/global.less";

/* START "Calendar.less" */
/* ======================================== */
/* CSS for control sap.ui.unified/Calendar  */
/* Evening Horizon theme                    */
/* ======================================== */

@_sap_ui_unified_Calendar_ItemColor: @sapUiLegendWorkingBackground;
@_sap_ui_unified_Calendar_ItemHoverColor: @sapUiListHoverBackground;
@_sap_ui_unified_Calendar_ItemBorderColor: @sapUiListBackground;
@_sap_ui_unified_Calendar_ItemWeekEndColor: @sapUiLegendNonWorkingBackground;
@_sap_ui_unified_Calendar_ItemWeekEndHoverColor: @sapUiListHoverBackground;
@_sap_ui_unified_Calendar_ItemOtherMonthColor: @sapUiContentLabelColor;

.sapUiCal {
	border-radius: 0.5rem;
}

.sapUiCalItem,
.sapUiCalWeekNum {
	height: 2.875rem;
}

.sapUiSizeCompact .sapUiCalSecType {
	.sapUiCal:not(.sapUiCalInt),
	&.sapUiCal:not(.sapUiCalInt) {
		min-width: 19rem;
	}
}

.sapUiSizeCompact .sapUiCalMonthSecType {
	.sapUiCalItem,
	.sapUiCalWeekNum {
		min-width: 2.25rem;
		height: 2.25rem;
	}
}

.sapUiCalWeekNum:hover {
	background-color: @sapUiListHoverBackground;
	border-radius: 0.5rem;
}

/* calendar should have only a border if on popup */
.sapUiCal[data-sap-ui-popup] {
	border-color: @sapUiContentForegroundBorderColor;
}

html[data-sap-ui-browser^="ff"].sap-desktop .sapUiCalHead > .sapUiCalHeadB:focus {
	outline-offset: 0; /* to hide firefox bug with native outline always displayed */
}

.sapUiCalItem[class*=sapUiCalItemType] {
	&.sapUiCalItemSel{
		&.sapUiCalItemNow {
			.sapUiCalItemText {
				border-radius: 0.3125rem;
				border-bottom-right-radius: 0.3125rem;
				border-bottom-left-radius: 0.3125rem;
			}
		}
	}
}

html.sap-desktop .sapUiCalHead > .sapUiCalHeadB:focus::before {
	content: "";
	position: absolute;
	box-sizing: border-box;
	left: 0.125rem;
	top: 0.125rem;
	right: 0.125rem;
	bottom: 0.125rem;
	border: 0.125rem solid @sapUiContentFocusColor;
	pointer-events: none;
	border-radius: 0.375rem;
}

.sapUiCalHeadB {
	position: relative;
}

.sapUiCalHead.sapUiCalHeaderWithTodayButton .sapUiCalHeadToday {
	position: relative;
}

html.sap-desktop .sapUiCalHead > .sapUiCalHeadB:focus:active {
	border-color: @sapUiContentContrastFocusColor;
}

.sapUiCalHead > button,
.sapUiCalCancel {
	font-family: @sapFontBoldFamily;
	color: @sapUiButtonLiteTextColor;
}

.sapUiCalSecType .sapUiCalHead > button .sapUiCalHeadBAddText {
	font-family: @sapUiFontFamily;
}

html.sap-desktop .sapUiCalHead > button:hover,
html.sap-desktop .sapUiCalCancel:hover,
html.sap-desktop .sapUiCalHead > button:focus,
html.sap-desktop .sapUiCalCancel:focus {
	color: @sapUiButtonLiteTextColor;
}

.sapUiCalWH,
.sapUiCalWeekNum {
	color: @sapUiContentLabelColor;
	font-size: @sapMFontSmallSize;
}

html[data-sap-ui-browser^="ie"].sap-desktop,
html[data-sap-ui-browser^="ed"].sap-desktop {
	.sapUiCalHead > .sapUiCalHeadB{
		position: relative;
		outline: none;
	}

	.sapUiCalHead > .sapUiCalHeadB:focus:after {
		position: absolute;
		top: 0.125rem;
		left: 0.125rem;
		right: 0.125rem;
		bottom: 0.125rem;
		border: 0.0625rem dashed @sapUiContentFocusColor;
		content: "";
	}

	.sapUiCalHead > .sapUiCalHeadB:focus:active:after {
		border-color: @sapUiContentContrastFocusColor;
	}
}

html.sap-desktop .sapUiCalHead > .sapUiCalHeadB:focus:not(:hover):not(:active) {
	background-color: transparent;
}

.sapUiCalHead > .sapUiCalHeadB,
.sapUiCalCancel,
html.sap-desktop .sapUiCalHead > .sapUiCalHeadB:hover,
html.sap-desktop .sapUiCalHead > .sapUiCalHeadB:focus,
.sapUiCalHead .sapUiCalHeadPrev,
.sapUiCalHead .sapUiCalHeadNext,
.sapUiCalHead .sapUiCalHeadToday {
	border-radius: 0.5rem;

}

.sapUiCalHead .sapUiCalHeadPrev:hover,
.sapUiCalHead .sapUiCalHeadNext:hover,
html.sap-desktop .sapUiCalHead > .sapUiCalHeadB:hover {
	box-shadow: 0 0 0.125rem 0 fade(@sapContent_ShadowColor, 72);
}

html.sap-desktop .sapUiCalHead > .sapUiCalHeadB:active {
	color: @sapUiButtonActiveTextColor;
	border-color: @sapUiButtonLiteActiveBorderColor;
}

/*** days, months, years ***/
.sapUiCalWH,
.sapUiCalItem,
.sapUiCalItemSelBetween.sapUiCalItemSel {
	border: 0.0625rem solid @_sap_ui_unified_Calendar_ItemBorderColor;
}

.sapUiCalItem {
	background-color: @sapUiLegendWorkingBackground;
	border-radius: 0.5rem;
}

.sapUiCalRowWeekNumber {
	color: @sapUiBaseText;
	background-color: @sapUiListBackground;
	border-color: @sapUiListBorderColor;
	border-radius: 0.5rem;
}

.sapUiCalItemNow {
	color: @sapUiBaseText;
}

.sapUiCalYearPicker,
.sapUiCalMonthPicker {
	.sapUiCalItem {
		outline: 0.0625rem solid @sapUiButtonLiteBorderColor;
		outline-offset: -0.125rem;
		background: @sapUiButtonLiteBackground;
		color: @sapUiButtonLiteTextColor;

		&:hover {
			outline: 0.0625rem solid @sapUiButtonLiteHoverBorderColor;
			color: @sapUiButtonLiteHoverTextColor;

			&.sapUiCalItemSel,
			&.sapUiCalItemSelBetween,
			&.sapUiCalItemSel:not(.sapUiCalItemSelBetween):not(.sapUiCalItemSelStart):not(.sapUiCalItemSelEnd) {
				outline: 0.0625rem solid @sapUiButtonSelectedHoverBorderColor;
				background: @sapUiButtonSelectedHoverBackground;
			}
		}

		&.sapUiCalItemSel,
		&.sapUiCalItemSelBetween,
		&.sapUiCalItemSel:not(.sapUiCalItemSelBetween):not(:hover) {
			outline: 0.0625rem solid @sapUiButtonSelectedBorderColor;
			background: @sapUiButtonSelectedBackground;
			color: @sapUiButtonSelectedTextColor;
		}
	}
}

/* make outline for days on number to prevent it around week number */
/* hide outline for all devices, but enable later for desktop */
.sapUiCalItem:focus {
	outline: none;
	overflow: hidden;
}

html.sap-desktop .sapUiCalItem.sapUiCalItemNow:not(.sapUiCalItemSelBetween):not(.sapUiCalItemSelStart):not(.sapUiCalItemSelEnd):focus > .sapUiCalItemText{
	outline-offset: -0.45rem;
}

html.sap-desktop {
	.sapUiCalMonthPicker,
	.sapUiCalYearPicker {
		.sapUiCalItem {
			&:focus::before {
				content: '';
				position: absolute;
				box-sizing: border-box;
				left: 0.125rem;
				top: 0.125rem;
				right: 0.125rem;
				bottom: 0.125rem;
				border: 0.125rem solid @sapUiContentFocusColor;
				border-radius: 0.375rem;
			}

			&.sapUiCalItemSel,
			&.sapUiCalItemSelBetween {
				&:focus:not(:hover) {
					background: @sapUiButtonSelectedBackground;
				}
			}
		}
	}
}

html.sap-desktop .sapUiCalItemSel:not(.sapUiCalItemNow):not(.sapUiCalItemSelBetween):focus > .sapUiCalItemText {
	color: @sapUiContentSelectedTextColor;
}

html.sap-desktop .sapUiSizeCompact .sapUiCalItemSel:not(.sapUiCalItemNow):not(.sapUiCalItemSelBetween):focus > .sapUiCalDayName {
	outline-color: @sapUiContentContrastFocusColor;
}

html[data-sap-ui-browser^="ie"].sap-desktop,
html[data-sap-ui-browser^="ed"].sap-desktop {
	// do this only for the Planing Calendar
	.sapMPlanCal :not(.sapUiCalFirstWDay).sapUiCalItem:focus {
		overflow: hidden;
	}

	.sapUiCalItem:focus > .sapUiCalItemText,
	.sapUiCalMonthPicker .sapUiCalItem:focus,
	.sapUiCalYearPicker .sapUiCalItem:focus {
		position: relative;
		outline: none;
	}

	.sapUiCalItem:focus > .sapUiCalItemText:after,
	.sapUiCalMonthPicker .sapUiCalItem:focus:after,
	.sapUiCalYearPicker .sapUiCalItem:focus:after {
		position: absolute;
		top: 0.125rem;
		left: 0.125rem;
		right: 0.125rem;
		bottom: 0.125rem;
		border: 0.0625rem dashed @sapUiContentFocusColor;
		content: "";
	}

	.sapUiCalItemSel:not(.sapUiCalItemNow):not(.sapUiCalItemSelBetween):focus > .sapUiCalItemText:after,
	.sapUiCalMonthPicker .sapUiCalItemSel:focus:after,
	.sapUiCalYearPicker .sapUiCalItemSel:focus:after {
		border-color: @sapUiContentContrastFocusColor;
	}
	.sapUiCalItemSel.sapUiCalItem:focus > .sapUiCalItemText {
		position: static;
	}

	.sapUiCalItem.sapUiCalItemNow:focus > .sapUiCalItemText:after {
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
	}
}

.sapUiCalItemWeekEnd,
html.sap-desktop .sapUiCalItemWeekEnd:not(.sapUiCalItemSel):focus {
	background-color: @sapUiLegendNonWorkingBackground;
}

.sapUiCalItem:hover,
.sapUiCalItemWeekEnd:hover {
	background-color: @sapUiListHoverBackground;
}

html.sap-desktop .sapUiCalDatesRow .sapUiCalItemWeekEnd:not(.sapUiCalItemSel):hover,
html.sap-desktop .sapUiCal:not(.sapUiCalDateInt) .sapUiCalItemWeekEnd:not(.sapUiCalItemSel) :not(.sapUiCalWeekNum):hover {
	background-color: @sapUiListHoverBackground;
}

html.sap-desktop .sapUiCalDatesRow .sapUiCalItemWeekEnd:not(.sapUiCalItemSel):hover {
	color: inherit;
}

html.sap-desktop .sapUiCalRow .sapUiCalItemSel:not(.sapUiCalItemNow):not(.sapUiCalItemSelBetween):focus > .sapUiCalItemText {
	border-top-left-radius: 0.25rem;
	border-bottom-left-radius: 0.25rem;
}

.sapUiCalItemNow:not(.sapUiCalItemSel):not(.sapUiCalItemSelBetween) > .sapUiCalItemText {
	box-shadow: inset 0 0 0 0.1875rem @_sap_ui_unified_Calendar_ItemBorderColor;
}

html.sap-desktop .sapUiCalItemSel:not(.sapUiCalItemNow):not(.sapUiCalItemSelBetween):focus > .sapUiCalDayName {
	border-top-left-radius: 0.25rem;
	border-bottom-left-radius: 0.25rem;
	border-top-right-radius: 0;
	border-bottom-right-radius: 0;
}

.sapUiSizeCompact .sapUiCalItem:not(.sapUiCalItemNow) {
	&.sapUiCalItemSelBetween .sapUiCalItemText {
		position: static;
	}
}
.sapUiSizeCompact .sapUiCalMonthSecType .sapUiCalItem {
	&.sapUiCalItemSelBetween .sapUiCalItemText {
		position: relative;
		line-height: 1rem;
	}
}

.sapUiSizeCompact .sapUiCalHead {
	height: 2.25rem;
}

.sapUiCalItem {
	position: relative;
	&:focus > .sapUiCalItemText {
		position: static;
		outline: none;
		&:after {
			position: absolute;
			top: 0.3125rem;
			left: 0.3125rem;
			right: 0.3125rem;
			bottom: 0.3125rem;
			content: "";
			border: 0.0625rem solid @sapUiContentFocusColor;
			border-radius: 0.5rem;
		}
	}
	&.sapUiCalItemSelBetween {
		.sapUiCalItemText {
			position: relative;
		}
		&:not(.sapUiCalItemNow) {
			.sapUiCalItemText:before {
				position: absolute;
				top: 0;
				left: 0;
				right: 0;
				bottom: 0;
				border: 0.0625rem solid @sapUiListSelectionBorderColor;
				content: "";
				border-radius: 0.5rem;
			}
		}
	}

	&.sapUiCalItemNow {
		border-radius: 0.5rem;
		&:hover {
			background: @sapUiListHoverBackground;
		}
		.sapUiCalSpecialDate,
		&:focus:not(.sapUiCalItemSel) .sapUiCalSpecialDate {
			width: 80%;
			bottom: 0.1875rem;
			left: 0.1875rem;
		}
		&.sapUiCalItemSel:not(.sapUiCalItemSelBetween),
		&.sapUiCalItemSelStart:not(.sapUiCalItemSelBetween),
		&.sapUiCalItemSelEnd:not(.sapUiCalItemSelBetween) {

			.sapUiCalItemText:before {
				position: absolute;
				top: 0.1875rem;
				left: 0.1875rem;
				right: 0.1875rem;
				bottom: 0.1875rem;
				border: 0.0625rem solid @sapUiListSelectionBorderColor;
				content: "";
				border-radius: 0.1875rem;
			}
		}
		&.sapUiCalItemSel:not(:focus) {
			.sapUiCalSpecialDate {
				width: 78%;
				bottom: 0.1875rem;
				left: 0.1875rem;
			}
		}
		&.sapUiCalItemSelBetween {
			line-height: normal;
			.sapUiCalItemText {
				border: 0.0625rem solid @sapUiListSelectionBorderColor;
				border-radius: 0.3125rem;
				margin: 0.125rem;
				width: 2.0625rem;
				height: 2.375rem;
				line-height: 2.375rem;
			}
			.sapUiCalSpecialDate {
				width: 78%;
			}
		}
		&:focus {
			&.sapUiCalItemSel,
			&.sapUiCalItemSelBetween:not(.sapUiCalItemSel),
			&:not(.sapUiCalItemSelBetween):not(.sapUiCalItemSel) {
				.sapUiCalSpecialDate {
					width: 66%;
					left: 0.3125rem;
					bottom: 0.3125rem;
					height: 0.3125rem;
				}
			}
		}

		.sapUiCalItemText {
			border: none;
		}
		&.sapUiCalItemSel {
			box-shadow: inset 0 0 0 0.25rem @sapUiListBackground;
		}
		&:focus > .sapUiCalItemText:before {
			top: 0.1875rem;
			left: 0.1875rem;
			right: 0.1875rem;
			bottom: 0.1875rem;
			border-radius: 0.3125rem;
		}
		&:before {
			position: absolute;
			content: "";
			top: 0;
			left: 0;
			right: 0;
			bottom: 0;
			border: 0.125rem solid @sapUiLegendCurrentDateTime;
			border-radius: 0.5rem;
			pointer-events: none;
		}
	}
	.sapUiCalItemText:after {
		outline: none;
	}
	.sapUiCalSpecialDate {
		position: absolute;
		width: 100%;
		height: 0.3125rem;
		bottom: 0.025rem;
		border-bottom-left-radius: 0.25rem;
		border-bottom-right-radius: 0.25rem;
		pointer-events: none;
	}

	&.sapUiCalItemSelBetween:not(.sapUiCalItemSel):not(.sapUiCalItemNow) {
		.sapUiCalSpecialDate {
			width: 91%;
			left: 0.0625rem;
			right: 0.0625rem;
			bottom: 0.025rem;
		}
	}

	&.sapUiCalItemSel .sapUiCalSpecialDate {
		bottom: 0.025rem;
		left: 0.09375rem;
		width: 90%;
	}

	.sapUiCalItemText {
		border-radius: 0.5rem;
	}
}

.sapUiSizeCozy {
	.sapUiCalItem {
		&:focus .sapUiCalSpecialDate,
		&.sapUiCalItemSel .sapUiCalSpecialDate,
		&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) .sapUiCalSpecialDate {
			width: 91%;
		}

		&.sapUiCalItemSelBetween {
			&:not(.sapUiCalItemSel):not(.sapUiCalItemNow) {
				.sapUiCalSpecialDate {
					width: 93%;
					left: 0.0625rem;
					right: 0.0625rem;
					bottom: 0.025rem;
				}
			}
		}
		&.sapUiCalItemNow {
			.sapUiCalSpecialDate,
			&.sapUiCalItemSel .sapUiCalSpecialDate {
				width: 82%;
			}

			&.sapUiCalItemSelBetween {
				.sapUiCalItemText {
					border: none;
					margin: auto;
					height: 100%;
					line-height: 2.875rem;
				}
				.sapUiCalItemText:before {
					content: "";
					position: absolute;
					top: 0.0625rem;
					left: 0.0625rem;
					right: 0.0625rem;
					bottom: 0.0625rem;
					border: 0.0625rem solid @sapUiListSelectionBorderColor;
					border-radius: 0.3125rem;
					margin: 0.125rem;
					line-height: 2.375rem;
				}
				.sapUiCalSpecialDate {
					left: 0.25rem;
					right: 0.25rem;
					width: 76%;
				}
			}
			&:focus {
				&:not(.sapUiCalItemSelBetween) {
					.sapUiCalSpecialDate {
						bottom: 0.375rem;
						width: 69%;
						left: 0.3125rem;
					}
				}
				.sapUiCalSpecialDate,
				&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) .sapUiCalSpecialDate{
					bottom: 0.375rem;
					width: 66%;
					left: 0.375rem;
				}
			}
		}
	}
}

.sapUiCalItemOtherMonth {
	background-color: @sapUiListBackground;
	color: @_sap_ui_unified_Calendar_ItemOtherMonthColor;
}

.sapUiCalMonthSecType {
    .sapUiCalItemOtherMonth.sapUiCalItem {
        &:not(.sapUiCalItemSelBetween),
        &:not(.sapUiCalItemSel) {
            .sapUiCalItemSecText {
                color: @_sap_ui_unified_Calendar_ItemOtherMonthColor;
            }
        }
    }
}

.sapUiCalItemOtherMonth.sapUiCalItemDsbl {
	opacity: @sapUiContentDisabledOpacity;
}

.sapUiCalItemSel {
	color: @sapUiContentContrastTextColor;
	&:focus > .sapUiCalItemText {
		&:after {
			top: -0.01rem;
			left: 0.025rem;
			right: 0.025rem;
			bottom: 0;
			border-radius: 0.5rem;
		}
	}
}

html.sap-desktop .sapUiCalItemOtherMonth:not(.sapUiCalItemSel) :not(.sapUiCalWeekNum):hover {
	background-color: @sapUiListHoverBackground;
}

.sapUiCalItem.sapUiCalItemSel:not(.sapUiCalItemSelBetween):not(:hover) {
	background-color: @sapUiContentSelectedBackground;
	color: @sapUiContentSelectedTextColor;
}

.sapUiCalItem.sapUiCalItemSel {
	background-color: @sapUiContentSelectedHoverBackground;
	&:not(.sapUiCalItemSelBetween):not(.sapUiCalItemSelStart):not(.sapUiCalItemSelEnd):hover {
		background-color: @sapUiListHoverBackground;
	}
}

.sapUiCalItemWeekEnd:hover {
	filter: brightness(105%);
}

.sapUiCalMonthPicker,
.sapUiCalYearPicker {
	.sapUiCalItem.sapUiCalItemDsbl {
		opacity: @sapUiContentDisabledOpacity;
	}
	.sapUiCalItem:focus {
		line-height: 2.875rem;
	}
}

.sapUiSizeCompact {
	.sapUiCalMonthPicker,
	.sapUiCalYearPicker {
		.sapUiCalItem:focus {
			line-height: 1.875rem;
		}
	}
}

.sapUiCalItemSel:not(.sapUiCalItemSelBetween) > .sapUiCalItemText {
	color: @sapUiContentSelectedTextColor;
	font-weight: bold;
	position: absolute;
	left: 0;
}

.sapUiCalItemSel.sapUiCalItemNow:not(.sapUiCalItemSelBetween) > .sapUiCalItemText {
	top: 0;
}

.sapUiCalItemSel {
	color: @sapContent_Selected_TextColor;
}

.sapUiCalItem:focus:not(.sapUiCalItemSelBetween){
	&:not(.sapUiCalItemSel) {
		.sapUiCalSpecialDate {
			width: 100%;
			left: 0;
			bottom: 0.025rem;
		}
	}
}

.sapUiCalItem:focus {
	&:not(.sapUiCalItemNow):not(.sapUiCalItemSel) .sapUiCalItemText:after {
		position: absolute;
		content: "";
		top: 0.3125rem;
		left: 0.3125rem;
		right: 0.3125rem;
		bottom: 0.3125rem;
		border: 0.125rem solid @sapUiContentFocusColor;
		border-radius: 0.1875rem;
	}
	&.sapUiCalItemNow,
	&.sapUiCalItemSel,
	&.sapUiCalItemSelStart,
	&.sapUiCalItemSelEnd {
		.sapUiCalItemText:after {
			content: "";
			position: absolute;
			top: 0.3125rem;
			left: 0.3125rem;
			right: 0.3125rem;
			bottom: 0.3125rem;
			border: 0.125rem solid @sapUiContentFocusColor;
			border-radius: 0.1875rem;
		}
	}
}

.sapUiCalMonthSecType {
	.sapUiCalItem {
		border-radius: 0.375rem;

		.sapUiCalItemSecText {
			width: 60%;
		}
		.sapUiCalSpecialDate,
		&.sapUiCalItemNow:not(:focus) .sapUiCalSpecialDate,
		&.sapUiCalItemNow.sapUiCalItemSelBetween:focus .sapUiCalSpecialDate,
		&.sapUiCalItemNow.sapUiCalItemSel:focus .sapUiCalSpecialDate,
		&.sapUiCalItemSel .sapUiCalSpecialDate,
		&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) .sapUiCalSpecialDate,
		&:focus:not(.sapUiCalItemSel):not(.sapUiCalItemSelBetween) .sapUiCalSpecialDate {
			width: 0.375rem;
			height: 0.375rem;
			bottom: 0.375rem;
			left: 0.375rem;
			border-radius: 50%;
		}

		&.sapUiCalItemSel,
		&.sapUiCalItemSelStart,
		&.sapUiCalItemSelEnd,
		&:not(.sapUiCalItemNow):not(.sapUiCalItemSel),
		&.sapUiCalItemSelBetween.sapUiCalItemOtherMonth:not(.sapUiCalItemNow) {
			.sapUiCalItemText:before {
				border-radius: 0.375rem;
			}
		}

		&.sapUiCalItemNow {
			&:before,
			.sapUiCalItemText {
				border-radius: 0.375rem;
			}
			&.sapUiCalItemSel:not(.sapUiCalItemSelBetween),
			&.sapUiCalItemSelStart:not(.sapUiCalItemSelBetween),
			&.sapUiCalItemSelEnd:not(.sapUiCalItemSelBetween),
			&.sapUiCalItemSelBetween.sapUiCalItemOtherMonth:not(.sapUiCalItemNow) {
				.sapUiCalItemText:before {
					border-radius: 0.1875rem;
				}
			}
		}
		&:focus {
			&:not(.sapUiCalItemNow):not(.sapUiCalItemSel) {
				.sapUiCalItemText:after {
					top: 0.0625rem;
					left: 0.0625rem;
					right: 0.0625rem;
					bottom: 0.0625rem;
					border: 0.125rem solid @sapUiContentFocusColor;
					border-radius: 0.25rem;
				}
			}
			&.sapUiCalItemSel,
			&.sapUiCalItemSelStart,
			&.sapUiCalItemSelEnd,
			&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) {
				.sapUiCalItemText:after {
					top: 0.125rem;
					left: 0.125rem;
					right: 0.125rem;
					bottom: 0.125rem;
					border: 0.125rem solid @sapUiContentFocusColor;
					border-radius: 0.25rem;
				}
			}
			&.sapUiCalItemNow {
				.sapUiCalItemText:after {
					top: 0.1875rem;
					left: 0.1875rem;
					right: 0.1875rem;
					bottom: 0.1875rem;
					border: 0.125rem solid @sapUiContentFocusColor;
					border-radius: 0.25rem;
				}
				.sapUiCalItemText:before {
					border: none;
				}

				&.sapUiCalItemSel,
				&.sapUiCalItemSelStart,
				&.sapUiCalItemSelEnd,
				&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) {
					.sapUiCalItemText:after {
						top: 0.3125rem;
						left: 0.3125rem;
						right: 0.3125rem;
						bottom: 0.3125rem;
						border: 0.125rem solid @sapUiContentFocusColor;
						border-radius: 0.25rem;
					}
					.sapUiCalItemText:before {
						border: 0.063rem solid @sapUiListSelectionBorderColor;
						border-radius: 0.1875rem;
					}
					.sapUiCalItemSecText {
						padding-right: 1.25rem;
    					padding-bottom: 1.15rem;
						text-align: right;
					}

					.sapUiCalSpecialDate {
						bottom: 0.5rem;
						left: 0.438rem;
					}
				}
			}
		}
	}
}

.sapUiCalItem.sapUiCalItemSel.sapUiCalItemNow:not(.sapUiCalItemSelBetween) > .sapUiCalItemText:before {
	top: 0.1875rem;
	left: 0.1875rem;
	right: 0.1875rem;
	bottom: 0.1875rem;
	border-radius: 0.3125rem;
}

.sapUiSizeCozy {
	.sapUiCalMonthSecType {
		.sapUiCalItem {
			&.sapUiCalItemSel,
			&.sapUiCalItemNow:focus:not(.sapUiCalItemSelBetween),
			&.sapUiCalItemSelBetween:not(.sapUiCalItemSel):not(.sapUiCalItemNow),
			&.sapUiCalItemNow.sapUiCalItemSelBetween:not(.sapUiCalItemSel) {
				.sapUiCalSpecialDate {
					width: 0.375rem;
					height: 0.375rem;
					bottom: 0.375rem;
					left: 0.375rem;
					border-radius: 50%;
				}
			}

			&.sapUiCalItemNow {
				&:focus {
					&.sapUiCalItemSel,
					&.sapUiCalItemSelStart,
					&.sapUiCalItemSelEnd,
					&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) {
						.sapUiCalSpecialDate {
							bottom: 0.5rem;
							left: 0.438rem;
						}
					}
				}
				&.sapUiCalItemSelBetween {
					.sapUiCalItemText {
						line-height: normal;
						width: 100%;
					}
					.sapUiCalItemText:before {
						top: 0.0625rem;
						left: 0.0625rem;
						right: 0.0625rem;
						bottom: 0.0625rem;
						border: 0.0625rem solid @sapUiListSelectionBorderColor;
						border-radius: 0.1875rem;
					}
					.sapUiCalItemText:after {
						border: 0.125rem solid @sapUiContentFocusColor;
						border-radius: 0.25rem;
					}
				}
			}
		}
	}
}

.sapUiSizeCompact {
	.sapUiCalItem {
		&.sapUiCalItemNow:focus {
			&.sapUiCalItemSelBetween,
			&.sapUiCalItemSelBetween.sapUiCalItemSel {
				.sapUiCalItemText:after {
					top: 0.0625rem;
					left: 0.0625rem;
					right: 0.0625rem;
					bottom: 0.0625rem;
					border: 0.125rem solid @sapUiContentFocusColor;
					border-radius: 0.1875rem;
				}
			}
		}
	}
}

.sapUiCalNoNameLine .sapUiCalItem.sapUiCalItemSel:not(.sapUiCalItemSelBetween) > .sapUiCalDayName {
	color: @sapUiContentSelectedTextColor
}

.sapUiCalItemSel.sapUiCalItemOtherMonth {
	background-color: @sapUiSelected;
}

.sapUiCalItemSel:not(.sapUiCalItemNow),
.sapUiCalItemSelStart:not(.sapUiCalItemNow),
.sapUiCalItemSelEnd:not(.sapUiCalItemNow),
.sapUiCalItemSelBetween:not(.sapUiCalItemNow),
.sapUiCalItemSelBetween.sapUiCalItemOtherMonth:not(.sapUiCalItemNow) {
	background-color: @sapUiListSelectionBackgroundColor;
	color: @sapTextColor;

	.sapUiCalItemText:before {
		position: absolute;
		top: 0;
		left: 0;
		right: 0;
		bottom: 0;
		border: 0.0625rem solid @sapUiListSelectionBorderColor;
		content: "";
		border-radius: 0.5rem;
	}
}

.sapUiCalItemSelBetween:not(.sapUiCalItemSel).sapUiCalItemNow,
.sapUiCalItemSel.sapUiCalItemSelBetween.sapUiCalItemNow {
	background-color: @sapUiListSelectionBackgroundColor;
}

html[data-sap-ui-browser^="ie"].sap-desktop .sapUiCalMonthPicker .sapUiCalItemSel:focus,
html[data-sap-ui-browser^="ie"].sap-desktop .sapUiCalYearPicker .sapUiCalItemSel:focus {
	/* as IE don't support outline-offset */
	border-color: @sapUiContentContrastFocusColor;
}

html.sap-desktop .sapUiCalItemSelBetween:hover {
	background-color: @sapUiListSelectionHoverBackground;
}

html.sap-desktop .sapUiCalItemSelBetween:focus:not(:hover){
	background-color: @sapUiListSelectionBackgroundColor;
}

.sapUiCalItemSel.sapUiCalItemNow:not(.sapUiCalItemSelBetween) > .sapUiCalItemText {
	border-radius: 0.5rem;
}

.sapUiCalItemNow.sapUiCalItemSelBetween{
	color: @sapUiBaseText;
}

.sapUiCalContentOver{
	bottom: 0.5rem;
}

/* Day names inside day */
.sapUiCalItemNow > .sapUiCalItemText > .sapUiCalItemTextAmPm{
	line-height: 2.5rem;
}

/* 2 calendar types */
.sapUiSizeCozy .sapUiCalMonthSecType .sapUiCalItem:not(.sapUiCalItemNow) .sapUiCalItemSecText {
	line-height: 0.75rem;
}

.sapUiCalItemNow:not(.sapUiCalItemSelBetween) > .sapUiCalItemText {
	line-height: 2.9rem;
}

.sapUiCalMonthSecType .sapUiCalItemNow:not(.sapUiCalItemSelBetween) > .sapUiCalItemText {
	line-height: normal;
}

.sapUiCalMonthSecType {
	.sapUiCalItem {
		.sapUiCalItemText {
			text-align: left;
			line-height: normal;
			padding-top: 0.1875rem;
			padding-left: 0.375rem;
		}
		.sapUiCalItemSecText {
			padding-right: 0.3125rem;
			text-align: right;
		}
		&.sapUiCalItemNow {
			&:focus.sapUiCalItemSel,
			&:focus.sapUiCalItemSelBetween:not(.sapUiCalItemSel) {
				.sapUiCalItemText {
					line-height: normal;
					padding-top: 0.3125rem;
					padding-left: 0.4375rem;
				}
				.sapUiCalItemSecText {
					padding-right: 0.4375rem;
				}
			}
		}
	}
}

.sapUiCalMonthSecType .sapUiCalItem.sapUiCalItemSel:not(.sapUiCalItemSelBetween) > .sapUiCalItemSecText {
	color: @sapUiContentSelectedTextColor;
}

/* Day names inside day */
.sapUiCalNoNameLine .sapUiCalItem > .sapUiCalItemText {
	padding-top: 0.75rem;
}

/*** compact mode (desktop size) ***/

.sapUiSizeCompact .sapUiCalItemNow:not(.sapUiCalItemSelBetween) > .sapUiCalItemText {
	line-height: 1.925rem;
}

.sapUiCalHead > button {
	height: 2.25rem;
}

.sapUiSizeCompact {
	.sapUiCalContentOver{
		bottom: 0.25rem;
	}
	/* Day names inside day */
	.sapUiCalItemNow > .sapUiCalItemText > .sapUiCalItemTextAmPm {
		line-height: 1.5rem;
	}
	.sapUiCalItemSelBetween .sapUiCalItemText,
	.sapUiCalItemSel.sapUiCalItemSelBetween:not(.sapUiCalItemNow) > .sapUiCalItemText {
		line-height: 1.75rem;
	}
	.sapUiCalMonthSecType {
		.sapUiCalItem {
			.sapUiCalItemText {
				text-align: left;
				line-height: normal;
				padding-top: 0.1875rem;
				padding-left: 0.3125rem;
			}

			.sapUiCalItemSecText {
				padding-right: 0.3125rem;
				text-align: right;
			}

			&.sapUiCalItemSelBetween > .sapUiCalItemText{
				line-height: normal;
			}

			&.sapUiCalItemNow {
				&:focus {
					&.sapUiCalItemSelBetween,
					&.sapUiCalItemSelBetween.sapUiCalItemSel,
					&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) {
						.sapUiCalItemText:after {
							border: 0.125rem solid @sapUiContentFocusColor;
							border-radius: 0.25rem;
						}
					}

					&.sapUiCalItemSel {
						.sapUiCalItemText {
							font-size: 0.75rem;
						}
						.sapUiCalSpecialDate {
							bottom: 0.5rem;
							left: 0.438rem;
						}
						&.sapUiCalItemSelBetween {
							.sapUiCalItemText {
								padding-top: 0.125rem;
								padding-left: 0.1875rem;
							}
						}
					}

					&.sapUiCalItemSelBetween:not(.sapUiCalItemSel) {
						.sapUiCalItemText {
							padding-top: 0.0625rem;
							padding-left: 0.1875rem;
							line-height: normal;
							font-size: @sapMFontSmallSize;
						}
						.sapUiCalItemText:after {
							top: 0.0625rem;
							left: 0.0625rem;
							right: 0.0625rem;
							bottom: 0.0625rem;
						}
					}
				}

				.sapUiCalItemText {
					line-height: normal;
				}
				&.sapUiCalItemSelBetween {
					.sapUiCalItemText {
						padding-top: 0.0625rem;
						padding-left: 0.0625rem;
						line-height: 0.75rem;
						width: 1.75rem;
						height: 1.75rem;
						border-radius: 0.1875rem;
					}
				}

				&.sapUiCalItemSel:not(.sapUiCalItemSelBetween){
					.sapUiCalItemText {
						line-height: normal;
					}
				}
			}
		}
	}

	.sapUiCalHead > button {
		margin: 0.1875rem 0;
		height: 2rem;
		.sapUiIcon {
			line-height: 2rem;
		}
	}
	.sapUiCalItem.sapUiCalItemNow.sapUiCalItemSelBetween > .sapUiCalItemText {
		width: 1.5rem;
		height: 1.5rem;
		top: 0.0625rem;
		line-height: 1.4375rem;
	}
}

@media only screen
and (orientation: portrait)
and (min-height: 380px) {
	.sap-phone .sapUiCal[data-sap-ui-popup] > .sapUiCalCancel {
		bottom: 1rem;
		left: 25%;
		width: 50%;
		padding-left: 1rem;
		padding-right: 1rem;
	}
}

.sapUiCalendarLandscape() {
	.sap-phone .sapUiCal[data-sap-ui-popup]{
		overflow: auto;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] > .sapUiCalHead {
		height: 1.875rem;
		padding-right: 5rem;
		box-sizing: border-box;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup].sapUiCalSecType > .sapUiCalHead {
		height: 2.875rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] > .sapUiCalContent {
		position: absolute;
		top: 2.875rem;
		left: 0.5rem;
		right: 0.5rem;
		bottom: 1rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup].sapUiCalSecType > .sapUiCalContent {
		top: 3.875rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalWH {
		height: 1.25rem;
		line-height: 1.25rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalHead > button {
		height: 100%;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalHeadPrev,
	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalHeadNext {
		padding-bottom: 0.5rem;
		box-sizing: border-box;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalHeadNext {
		right: 5.5rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalContent {
		.sapUiCalDatesRow,
		.sapUiCalMonthView {
			position: absolute;
			top: 0.25rem;
			left: 0;
			right: 0;
			bottom: 0;
			width: auto;
			margin: 1rem;
			min-height: 11.25rem;
		}
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] > .sapUiCalContent > .sapUiCalMonthPicker,
	.sap-phone .sapUiCal[data-sap-ui-popup] > .sapUiCalContent > .sapUiCalYearPicker {
		padding: 1.25rem 0.5rem 0 0.5rem;
		min-height: 11.25rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] {
		.sapUiCalDatesRow > div[role=row] > div,
		.sapUiCalMonthView > div[role=row] > div {
			height: 16.66%;
			min-height: 1.875rem;
		}
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalItem {
		height: 100%;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalItem,
	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalWeekNum {
		line-height: 2.1875rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalItemNow > .sapUiCalItemText {
		line-height: 1.9375rem;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] .sapUiCalWeekNum {
		height: 100%;
	}

	.sap-phone .sapUiCal[data-sap-ui-popup] > .sapUiCalCancel {
		top: 1rem;
		right: 1rem;
		width: 5rem;
		height: 1.875rem;
		padding-top: 0.25rem;
		padding-bottom: 0.5rem;
		box-sizing: border-box;
	}

.sap-phone .sapUiCal[data-sap-ui-popup].sapUiCalSecType > .sapUiCalCancel {
		top: 1.5rem;
		padding-top: 0.5rem;
	}
}

@media only screen
and (min-width : 320px)
and (max-height: 379px) {
	.sapUiCalendarLandscape();
}

@media only screen
and (min-width : 320px)
and (min-height: 380px)
and (orientation: landscape) {
	.sapUiCalendarLandscape();
}

/* colors for special days */
.dayColorMixIn(@class, @color){
	.@{class} > .sapUiCalSpecialDate {
		background-color: @color;
		border-bottom-left-radius: 0.5rem;
		border-bottom-right-radius: 0.5rem;
	}
	.@{class}.sapUiCalItemNow > .sapUiCalSpecialDate {
		box-sizing: border-box;
		height: 0.3125rem;
		border-bottom-left-radius: (@sapUiElementBorderCornerRadius / 2);
		border-bottom-right-radius: (@sapUiElementBorderCornerRadius / 2);
	}
	.@{class}.sapUiCalItemSel.sapUiCalItemNow .sapUiCalItemText {
		border-top-left-radius: (@sapUiElementBorderCornerRadius / 2);
		border-top-right-radius: (@sapUiElementBorderCornerRadius / 2);
	}
	html.sap-desktop .sapUiCal:not(.sapUiCalDateInt) .sapUiCalItem:not(.sapUiCalItemSel) {
		&.@{class} > .sapUiCalSpecialDate:hover {
			background-color: @color;
			border-bottom-right-radius: 0.25rem;
			border-bottom-left-radius: 0.25rem;
			border-top-left-radius: 0;
			border-top-right-radius: 0;
		}
		&.@{class}.sapUiCalItemNow > .sapUiCalSpecialDate:hover {
			border-bottom-right-radius: 0.125rem;
			border-bottom-left-radius: 0.125rem;
		}
	}
}

.sapUiCalTwoMonthsTwoColumnsJaZh {
	.sapUiCalHead > button.sapUiCalHeadB2 {
		margin-left: calc(~"25% - 6rem");
	}
}

.sapUiCalTwoMonthsTwoColumns {
	.sapUiCalHead > button.sapUiCalHeadB1 {
		margin-left: calc(~"25% - 6rem");
	}
}

.dayColorMixIn(sapUiCalItemType01, @_sap_ui_unified_Calendar_sapUiCalItemType01);
.dayColorMixIn(sapUiCalItemType02, @_sap_ui_unified_Calendar_sapUiCalItemType02);
.dayColorMixIn(sapUiCalItemType03, @_sap_ui_unified_Calendar_sapUiCalItemType03);
.dayColorMixIn(sapUiCalItemType04, @_sap_ui_unified_Calendar_sapUiCalItemType04);
.dayColorMixIn(sapUiCalItemType05, @_sap_ui_unified_Calendar_sapUiCalItemType05);
.dayColorMixIn(sapUiCalItemType06, @_sap_ui_unified_Calendar_sapUiCalItemType06);
.dayColorMixIn(sapUiCalItemType07, @_sap_ui_unified_Calendar_sapUiCalItemType07);
.dayColorMixIn(sapUiCalItemType08, @_sap_ui_unified_Calendar_sapUiCalItemType08);
.dayColorMixIn(sapUiCalItemType09, @_sap_ui_unified_Calendar_sapUiCalItemType09);
.dayColorMixIn(sapUiCalItemType10, @_sap_ui_unified_Calendar_sapUiCalItemType10);
.dayColorMixIn(sapUiCalItemType11, @_sap_ui_unified_Calendar_sapUiCalItemType11);
.dayColorMixIn(sapUiCalItemType12, @_sap_ui_unified_Calendar_sapUiCalItemType12);
.dayColorMixIn(sapUiCalItemType13, @_sap_ui_unified_Calendar_sapUiCalItemType13);
.dayColorMixIn(sapUiCalItemType14, @_sap_ui_unified_Calendar_sapUiCalItemType14);
.dayColorMixIn(sapUiCalItemType15, @_sap_ui_unified_Calendar_sapUiCalItemType15);
.dayColorMixIn(sapUiCalItemType16, @_sap_ui_unified_Calendar_sapUiCalItemType16);
.dayColorMixIn(sapUiCalItemType17, @_sap_ui_unified_Calendar_sapUiCalItemType17);
.dayColorMixIn(sapUiCalItemType18, @_sap_ui_unified_Calendar_sapUiCalItemType18);
.dayColorMixIn(sapUiCalItemType19, @_sap_ui_unified_Calendar_sapUiCalItemType19);
.dayColorMixIn(sapUiCalItemType20, @_sap_ui_unified_Calendar_sapUiCalItemType20);
/* END "Calendar.less" */

/* START "CalendarLegend.less" */
/* ============================================== */
/* CSS for control sap.ui.unified/CalendarLegend  */
/* Evening Horizon theme                          */
/* ============================================== */

.sapUiUnifiedLegend {
	border-radius: 0.5rem;
}

.sapUiCalLegDayTypeNonWorking,
.sapUiUnifiedLegendNonWorkingDay {
	& > .sapUiUnifiedLegendSquare > .sapUiUnifiedLegendSquareColor {
		background: @sapUiLegendNonWorkingBackground;
	}
}

.sapUiUnifiedLegendItem {
	position: relative;

	&:focus {
		outline: none;
		&::after {
			content: "";
			position: absolute;
			box-sizing: border-box;
			left: 0.125rem;
			top: 0.125rem;
			right: 0.125rem;
			bottom: 0.125rem;
			border: @sapContent_FocusWidth @sapContent_FocusStyle @sapUiContentFocusColor;
			border-radius: 0.25rem;
			pointer-events: none;
			top: 0;
			left: 0;
			bottom: 0;
			right: 0;
		}
	}
}

.sapUiSizeCompact .sapUiUnifiedLegendItems,
.sapUiSizeCompact.sapUiUnifiedLegendItems {
	position: relative;

	&:focus {
		outline: none;
		&::after {
			content: "";
			position: absolute;
			box-sizing: border-box;
			left: 0.025rem;
			top: 0.025rem;
			right: 0.025rem;
			bottom: 0.025rem;
			border: @sapContent_FocusWidth @sapContent_FocusStyle @sapUiContentFocusColor;
			border-radius: 0.25rem;
			pointer-events: none;
		}
	}
}


/* END "CalendarLegend.less" */

/* START "CalendarRow.less" */
/* =========================================== */
/* CSS for control sap.ui.unified/CalendarRow  */
/* Evening Horizon theme                       */
/* =========================================== */

/* intervals */
.sapUiCalendarRowAppsInt,
.sapUiCalendarRowAppsSubInt{
	border-color: @sapUiListVerticalBorderColor;
}

.sapUiCalendarRowAppsIntHead{
	font-size: @sapMFontSmallSize;
	color: @sapUiBaseText;
}

.sapMListTblRow:not(.sapMLIBSelected) .sapUiCalendarRowAppsNoWork,
.sapMListTblSubRow:not(.sapMLIBSelected) .sapUiCalendarRowAppsNoWork {
	background-color: @sapUiLegendNonWorkingBackground;
}

/* Appointments */
/* as outlines can't have different offsets on one side (no left border) and don't work in IE just use a inner border */
html.sap-desktop .sapUiCalendarApp:focus > .sapUiCalendarAppCont:after {
	position: absolute;
	top: 0.0625rem;
	left: 0.0625rem;
	right: 0.0625rem;
	bottom: 0.0625rem;
	content: "";
	border: 0.125rem solid @sapUiContentFocusColor;
	pointer-events: none;
	border-radius: 0.125rem;
}

html.sap-desktop .sapUiCalendarApp.sapUiCalendarAppSel:focus:hover > .sapUiCalendarAppCont:after {
	top: 0.125rem;
	left: 0.125rem;
	right: 0.125rem;
	bottom: 0.125rem;
}

.sapUiCalendarApp,
.sapUiCalendarAppCont {
	border-radius: 0.25rem;
}

.sapUiCalendarAppCont {
	border-top-left-radius: 0;
	border-bottom-left-radius: 0;
}

html.sap-desktop .sapUiCalendarApp:hover:not(:active),
html.sap-desktop .sapUiCalendarApp:focus:not(:active) > .sapUiCalendarAppCont{
	background-color: @sapUiListBackground;
}

html.sap-desktop .sapUiCalendarApp.sapUiCalendarAppSel:hover:not(:active),
html.sap-desktop .sapUiCalendarApp.sapUiCalendarAppSel:focus:not(:active) > .sapUiCalendarAppCont{
	background-color: @sapUiContentSelectedBackground;
}

.sapMListTblRow.sapMLIBFocusable:focus {
	.sapUiCalendarRowAppsInt.sapUiCalendarRowAppsNoWork:before {
		position: absolute;
		top: 0px;
		left: -1px;
		right: 1px;
		bottom: 0px;
		content: "";
		border: 0.195rem solid @sapUiContentFocusColor;
		border-left: none;
		pointer-events: none;
	}

	.sapUiCalendarRowAppsIntNoHead.sapUiCalendarRowAppsInt.sapUiCalendarRowAppsNoWork:before {
		position: absolute;
		top: 0px;
		left: -1px;
		right: 1px;
		bottom: 1px;
		content: "";
		border: 0.0625rem dotted @sapUiContentFocusColor;
		pointer-events: none;
	}
}

html[data-sap-ui-browser ^= "ie"], html[data-sap-ui-browser^="ed"] {
	.sapUiCalendarApp:focus > .sapUiCalendarAppCont:after{
		border-style: dashed;
	}
}

.sapUiCalendarApp.sapUiCalendarAppSel,
.sapUiCalendarApp:active,
.sapUiCalendarApp.sapUiCalendarAppSel:active{
	color: @sapUiContentSelectedTextColor;
	background-color:@sapUiContentSelectedBackground
}

html.sap-desktop .sapUiCalendarApp.sapUiCalendarAppSel:hover:not(:active),
html.sap-desktop .sapUiCalendarApp.sapUiCalendarAppSel:focus:not(:active){
	color: @sapUiContentSelectedTextColor;
}

.sapUiCalendarApp.sapUiCalendarAppSel > .sapUiCalendarAppCont,
.sapUiCalendarApp:active > .sapUiCalendarAppCont,
.sapUiCalendarApp.sapUiCalendarAppSel:active > .sapUiCalendarAppCont{
	border-color: @sapUiListSelectionBorderColor;
	border-top-left-radius: 0;
	border-bottom-left-radius: 0;
}


html.sap-desktop .sapUiCalendarApp.sapUiCalendarAppSel:focus:not(:active)  > .sapUiCalendarAppCont,
html.sap-desktop .sapUiCalendarApp.sapUiCalendarAppSel:hover:not(:active)  > .sapUiCalendarAppCont{
	border-color: @sapUiListSelectionBorderColor;
	border-left-color: @sapUiListBackground;
}

.sapUiCalendarApp.sapUiCalendarAppSel > .sapUiCalendarAppCont > .sapUiCalendarAppIcon,
.sapUiCalendarApp.sapUiCalendarAppSel > .sapUiCalendarAppCont > .sapUiCalendarAppArrowIconLeft,
.sapUiCalendarApp.sapUiCalendarAppSel > .sapUiCalendarAppCont > .sapUiCalendarAppArrowIconRight,
.sapUiCalendarApp.sapUiCalendarAppSel > .sapUiCalendarAppCont .sapUiCalendarAppTitle,
.sapUiCalendarApp.sapUiCalendarAppSel > .sapUiCalendarAppCont .sapUiCalendarAppText,
.sapUiCalendarApp.sapUiCalendarAppSel > .sapUiCalendarAppCont .sapUiCalendarAppDescription {
	color: @sapUiContentSelectedTextColor;
}

.sapUiCalendarAppTitle{
	font-size: @sapMFontMediumSize;
	color: @sapUiBaseText;
}

.sapUiCalendarAppText{
	font-size: @sapMFontSmallSize;
}

.sapUiCalendarRowNow{
	border-color: @sapUiCalendarColorToday;
}

.sapUiAppCreate.sapUiCalendarApp.sapUiCalendarAppType01 {
	border-top-color: darken(@sapUiListBorderColor, 15);
	border-right-color: darken(@sapUiListBorderColor, 15);
	border-bottom-color: darken(@sapUiListBorderColor, 15);
	background-color: lighten(@sapUiAccent1, 50);
	color: @sapUiListTextColor;
}

.sapUiCalendarApp.sapUiCalendarAppTent > .sapUiCalendarAppCont {
	border-top-left-radius: 0;
	border-bottom-left-radius: 0;
}

.sapUiCalendarApp.sapUiCalendarAppTent.sapUiCalendarAppSel > .sapUiCalendarAppCont,
.sapUiCalendarApp.sapUiCalendarAppTent > .sapUiCalendarAppCont:hover {
	border-style: solid;
	border-left: none;
}

.sapUiCalendarAppCont:hover {
	border-color: @sapUiListSelectionBorderColor;
}

// SPC
.sapUiCalendarAppSel:hover .sapUiCalendarAppCont,
// PC
.sapUiCalendarAppSel .sapUiCalendarAppCont:hover {
	box-shadow: inset 0 0 0 0.0625rem @sapUiListSelectionBorderColor;
}
/* END "CalendarRow.less" */

/* START "ColorPicker.less" */
/* =========================================== */
/* CSS for control sap.ui.unified/ColorPicker  */
/* Evening Horizon theme                       */
/* =========================================== */

.sapUiColorPicker-ColorPickerSlider.sapUiSliHori .sapUiSliBar,
.sapUiColorPicker-ColorPickerSlider.sapUiSliHori .sapUiSliBar .sapUiSliHiLi {
	border: none;
	background-color: transparent;
}

.sapUiColorPicker-ColorPickerAlphaSlider.sapUiSliHori .sapUiSliBar,
.sapUiColorPicker-ColorPickerAlphaSlider.sapUiSliHori .sapUiSliBar .sapUiSliHiLi {
	border: none;
	background-color: transparent;
}

.sapUnifiedColorPicker {
	.sapUiColorPicker-ColorPickerAlphaSlider,
	.sapUiColorPicker-ColorPickerSlider {
		.sapMSliderHandle {
			border: 0.125rem solid @sapUiFieldBorderColor;
			background-color: transparent;
			height: 1.625rem;
		}
	}
}

.sapUiColorPicker-ColorPickerMatrix.sapUnifiedColorPicker {
	.sapUiColorPicker-ColorPickerHexField,
	.sapUiCPHexWrapper,
	.sapUiCPComparisonWrapper {
		.sapUiColorPicker-ColorPickerOldColor,
		.sapUiColorPicker-ColorPickerNewColor {
			height: 2.25rem;
		}
		margin-top: 0.5rem;
		vertical-align: top;
	}
}

.sapUiColorPicker-Large.sapUiCPPhone .sapUiCPComparisonWrapper,
.sapUiColorPicker-ColorPickerMatrix.sapUnifiedColorPicker .sapUiCPHexWrapper,
.sapUiColorPicker-ColorPickerMatrix.sapUnifiedColorPicker:not(.sapUiCPPhone) .sapUiCPComparisonWrapper {
	height: 2.25rem;
}

.sapUiColorPicker-Large {
	.sapUiCPHSLVText {
		padding-left: 2.125rem;
	}
}

.sapUiColorPicker-Simplified.sapUnifiedColorPicker.sapUiCPPhone .sapUiColorPicker-ColorPickerNewColor,
.sapUiColorPicker-Simplified.sapUnifiedColorPicker.sapUiCPPhone .sapUiColorPicker-ColorPickerOldColor {
	height: 2.75rem;
}

.sapUiColorPicker-Default.sapUnifiedColorPicker.sapUiCPPhone .sapUiColorPicker-ColorPickerNewColor,
.sapUiColorPicker-Default.sapUnifiedColorPicker.sapUiCPPhone .sapUiColorPicker-ColorPickerOldColor {
	height: 2.3125rem;
}

.sapUiSizeCompact {
	.sapUnifiedColorPicker:not(.sapUiCPPhone) .sapUiCPComparisonWrapper {
		height: 1.5rem;
		vertical-align: top;
	}

	.sapUiColorPicker-Default.sapUnifiedColorPicker:not(.sapUiCPPhone) .sapMBtn .sapMBtnInner .sapMBtnIcon {
		margin-left: 0.3rem;
	}

	.sapUiColorPicker-Simplified.sapUnifiedColorPicker.sapUiCPPhone .sapUiColorPicker-ColorPickerNewColor,
	.sapUiColorPicker-Simplified.sapUnifiedColorPicker.sapUiCPPhone .sapUiColorPicker-ColorPickerOldColor {
		height: 2.065rem;
	}
	.sapUiColorPicker-Default.sapUnifiedColorPicker.sapUiCPPhone {
		.sapUiColorPicker-ColorPickerNewColor,
		.sapUiColorPicker-ColorPickerOldColor {
			width: 2.5rem;
			height: 1.5rem;
		}
		.sapMBtn {
			width: 2.5rem;
			margin-left: 1.1rem;
		}
		.sapUiColorPicker-ColorPickerSlider {
			margin-bottom: 0.0625rem;
		}
	}
	.sapUnifiedColorPicker {
		.sapUiColorPicker-ColorPickerAlphaSlider,
		.sapUiColorPicker-ColorPickerSlider {
			.sapMSliderHandle {
			  height: 1.25rem;
			}
		}
	}
}

.sapUiColorPicker-ColorPickerMatrix.sapUnifiedColorPicker.sapUiCPPhone .sapUiColorPicker-ColorPickerAlphaSlider {
	padding-top: 0.1875rem;
}

.sapUiColorPicker-ColorPickerMatrix.sapUnifiedColorPicker .sapUiCPPercentSymbol {
	line-height: 2.75rem;
}

.sapUnifiedColorPicker .sapMSliderInner:not(.sapMSliderInnerDisabled):hover .sapMSliderHandle {
	background: @sapUiButtonBackground;
	border-color: darken(@sapUiFieldBorderColor, 15);
}

.sapUnifiedColorPicker .sapMSliderInner::before,
.sapUnifiedColorPicker .sapMSlider::after {
	display: none;
}

.sapUnifiedColorPicker .sapMSliderInner .sapMSliderHandle:focus {
	outline: 0.1875rem solid @sapUiContentFocusColor;
}

.sapUnifiedColorPicker .sapMSliderInner .sapMSliderHandle:focus::before {
	content: "";
	outline: 0.0625rem solid @sapContent_ContrastFocusColor;
	outline-offset: -0.125rem;
}

.sapUnifiedColorPicker .sapMSliderInner .sapMSliderHandle:not(.sapMSliderHandlePressed):not(.sapMRangeSlider.sapMSlider .sapMSliderInner:not(.sapMSliderInnerDisabled):hover > .sapMSliderHandle):not(.sapMRangeSlider.sapMSlider .sapMSliderInner:not(.sapMSliderInnerDisabled) .sapMSliderHandle:focus):after {
	display: none;
}


.sapUnifiedColorPicker .sapMSliderInner:not(.sapMSliderInnerDisabled).sapMSliderPressed .sapMSliderHandle,
.sapUnifiedColorPicker .sapMSliderInner:not(.sapMSliderInnerDisabled) .sapMSliderHandle:focus {
	border: 0.125rem solid darken(@sapUiFieldBorderColor, 15);
	background-color: transparent;
}
/* END "ColorPicker.less" */

/* START "FileUploader.less" */
/* ============================================ */
/* CSS for control sap.ui.unified/FileUploader  */
/* Evening Horizon theme                        */
/* ============================================ */

.sapUiFup,
.sapUiFup.sapMBarChild {
	height: 2.75rem;
}

.sapUiSizeCompact{
	.sapUiFup,
	.sapUiFup.sapMBarChild {
		height: 2rem;
	}
}

/* increase outline offset */
.sapMIBar-CTX .sapUiFup .sapMBtnInner:focus.sapMFocusable {
	outline-offset: -.125rem;
}

.sapUiFup .sapMBtn:focus {
	outline: none;
}

html.sap-desktop .sapUiFup .sapMFocusable:focus,
html.sap-desktop .sapUiFup .sapMFocusable:active {
	outline: 1px dotted @sapUiContentFocusColor;
	outline-offset: -1px;
}

.sapMFooter-CTX .sapUiFup .sapMBtnInner:focus.sapMFocusable,
.sapMFooter-CTX .sapUiFup .sapMBtnInner:active.sapMFocusable,
.sapUiFup .sapMBtnInner:active.sapMBtnAccept,
html.sap-desktop .sapUiFup .sapMBtnInner:active.sapMBtnAccept,
.sapUiFup .sapMBtnInner:active.sapMBtnReject,
html.sap-desktop .sapUiFup .sapMBtnInner:active.sapMBtnReject,
.sapUiFup .sapMBtnInner:active.sapMBtnEmphasized,
html.sap-desktop .sapUiFup .sapMBtnInner:active.sapMBtnEmphasized {
	outline-color: @sapUiContentContrastFocusColor;
	outline-offset: -.125rem;
}

.sapMFooter-CTX .sapUiFup .sapMBtnInner:active.sapMFocusable {
	background-color: @sapUiButtonActiveBackground;
}

/* IE ignores outline-offset. Use an overlay: */

html[data-sap-ui-browser^="ie"] {

	.sapUiFup .sapMBtnInner:focus.sapMFocusable {
		outline: none;
		position: relative;
	}

	.sapUiFup .sapMBtnInner:focus.sapMFocusable:before {
		content: " ";
		box-sizing: border-box;
		width: 100%;
		height: 100%;
		position: absolute;
		left: 0;
		top: 0;
		border: 1px dotted @sapUiContentFocusColor;
	}

	.sapMFooter-CTX .sapUiFup .sapMBtnInner:focus.sapMFocusable:before {
		border-color: @sapUiContentContrastFocusColor;
	}

}
/* END "FileUploader.less" */

/* START "Menu.less" */
/* ==================================== */
/* CSS for control sap.ui.unified/Menu  */
/* Evening Horizon theme                */
/* ==================================== */

.sapUiMnu {
	border-radius: @sapUiPopoverBorderCornerRadius;
}

.sapUiMnuItmSbMnu > div:after {
	font-size: 1rem;
}

.sapUiMnuItm.sapUiMnuTfItm .sapUiMnuTfItemTf.sapUiMnuTfItemTfDsbl {
	opacity: @sapUiContentDisabledOpacity;
}

.sapUiMnuItm.sapUiMnuItmSubMnuOpen.sapUiMnuItmHov:hover {
	background-color: @sapUiListSelectionHoverBackground;
}

.sapUiMnu  .sapUiMnuDiv>hr,
.sapMList .sapMListItems .sapUiMnuDiv>hr{
	border-top: 0.0625rem solid @sapUiGroupContentBorderColor;
	inset-inline-start: 0rem;
	inset-inline-end: 0rem;
}

.sapUiMnu.sapUiMnuCozySupport.sapUiSizeCozy .sapUiMnuItm {
	height: 2.75rem;
	line-height: 2.75rem;
}

.sapUiMnu.sapUiMnuCozySupport.sapUiSizeCozy .sapUiMnuItmIco > .sapUiIcon {
	line-height: 2.75rem;
	font-size: 1rem;
}

.sapUiMnu.sapUiMnuCozySupport.sapUiSizeCozy .sapUiMnuItmSbMnu {
	width: 2.25rem;
	height: 2.75rem;
	font-size: 0.75rem;
	&.sapUiIconMirrorInRTL{
		height: 2.75rem;
	}
}

html.sap-phone .sapUiSizeCozy {
	.sapUiMnuLst > .sapUiMnuItm > .sapUiMnuItmIco + .sapUiMnuItmTxt {
		padding-inline-start: 0;
	}
}

.sapMMenuLIArrowRightIcon {
	font-size: 1rem;
	width: 2.75rem;
	height: 2.75rem;
	line-height: 2.75rem;
}

.sapUiSizeCompact .sapUiMnuItmR, .sapUiSizeCompact .sapUiMnuNoIco .sapUiMnuItmL {
	width: 0.5rem;
}

.sapUiMnu .sapUiMnuLst .sapUiMnuItm.sapUiMnuItmSubMnuOpen,
.sapUiMnu .sapUiMnuLst .sapUiMnuItm.sapUiMnuItmSubMnuOpen.sapUiMnuItmHov {
	border-bottom: 1px solid @sapUiListSelectionBorderColor;
}

.sapUiMnu .sapUiMnuLst .sapUiMnuItm {
	border-bottom: 1px solid transparent;
}

.sapUiMnu .sapUiMnuLst .sapUiMnuItm.sapUiMnuItmHov:active{
	border-bottom: 1px solid transparent;
}

.sapUiMnu .sapUiMnuLst .sapUiMnuItm.sapUiMnuItmSubMnuOpen.sapUiMnuItmHov:active{
	border-bottom: 1px solid transparent;
	background-color: @sapUiListActiveBackground;
}

.sapUiMnuItm:focus {
	outline: 0.125rem solid @sapUiContentFocusColor;
	outline-offset: -0.1875rem;
}

html.sap-desktop,
html.sap-tablet {
	.sapUiSizeCompact {
		.sapUiMnuLst > .sapUiMnuItm > .sapUiMnuItmIco + .sapUiMnuItmTxt {
			padding-inline-start: 0;
		}
	}
	.sapUiMnuLst > .sapUiMnuItm > .sapUiMnuItmIco + .sapUiMnuItmTxt {
		padding-inline-start: 0;
	}
}

.sapUiMnuItm.sapUiMnuTfItm .sapUiMnuTfItemTf {
	border-radius: 0.25rem;
}

.sapUiMnuItm.sapUiMnuTfItm .sapUiMnuTfItemTf:focus {
	border-width: 0.125rem;
}
/* END "Menu.less" */

/* START "ShellHeader.less" */
/* =========================================== */
/* CSS for control sap.ui.unified/ShellHeader  */
/* Evening Horizon theme                       */
/* =========================================== */

.sapUiUfdShellHeadItm:active > span,
.sapUiUfdShellHeadItm.sapUiUfdShellHeadItmSel > span,
.sapUiUfdShellHeadItm:active:hover > span,
html.sapUiMedia-Std-Desktop .sapUiUfdShellHeadUsrItm:hover:active > *,
.sapUiUfdShellHeadUsrItm:active > * {
	color: @sapUiToggleButtonPressedTextColor;
}

.sapUiUfdShellHeadUsrItm:focus .sapUiUfdShellHeadUsrItmExp,
.sapUiUfdShellHeadUsrItm:focus .sapUiUfdShellHeadUsrItmName,
.sapUiUfdShellHeadUsrItm:focus .sapUiUfdShellHeadUsrItmImg,
.sapUiUfdShellHeadItm:focus > span {
	border-color: @sapUiContentFocusColor;
}
/* END "ShellHeader.less" */

/* START "ShellLayout.less" */
/* =========================================== */
/* CSS for control sap.ui.unified/ShellLayout  */
/* Evening Horizon theme                       */
/* =========================================== */

@_sap_ui_unified_ShellLayout_CurtainBGStartColor: fade(@sapUiGroupContentBackground,97);
@_sap_ui_unified_ShellLayout_CurtainBGEndColor: fade(lighten(desaturate(@sapUiHighlight,37),56),97);

._sapUiUfdShellSubHdr { /* Testpage only */
	background-color: @sapUiShellBackground;
	color: @sapUiShellTextColor;
}

.sapUiUfdShell .sapUiUfdShellHead {
	background-color: @sapUiBlockLayerBackground;
}

.sapUiUfdShell .sapUiUfdShellBrand {
	background: @sapUiShellBackground;
}

.sapUiUfdShell.sapUiUfdShellCurtainVisible .sapUiUfdShellHead,
.sapUiUfdShell.sapUiUfdShellCurtainVisible .sapUiUfdShellHead > div {
	background: none;
	background-color: @sapUiShellBackground;
	filter: none;
}

.sapUiUfdShellCurtain {
	border-bottom: 1px solid @sapUiShellBorderColor;
	background-image: linear-gradient(to bottom, @_sap_ui_unified_ShellLayout_CurtainBGStartColor 0%, @_sap_ui_unified_ShellLayout_CurtainBGEndColor 100%);
	background-image: -webkit-linear-gradient(top, @_sap_ui_unified_ShellLayout_CurtainBGStartColor 0%, @_sap_ui_unified_ShellLayout_CurtainBGEndColor 100%);
	background-attachment: fixed;
	background-repeat: repeat, no-repeat;
}
/* END "ShellLayout.less" */

/* START "ShellOverlay.less" */
/* ============================================ */
/* CSS for control sap.ui.unified/ShellOverlay  */
/* Evening Horizon theme                        */
/* ============================================ */

.sapUiUfdShellOvrlyHeadCntnt {
	background-color: @sapUiShellBackground;
	box-shadow: inset 0 -1px @sapUiShellBorderColor;
}

.sapUiUfdShellOvrlyHeadClose,
.sapUiUfdShellOvrlyHeadClose:hover,
.sapUiUfdShellOvrlyHeadClose:focus,
.sapUiUfdShellOvrlyHeadClose:visited {
	color: @sapUiButtonIconColor;
	text-shadow: @sapUiShadowText;
}

.sapUiUfdShellOvrlyHeadClose:active {
	background-color: @sapUiButtonActiveBackground;
	color: @sapUiButtonActiveTextColor;
}
/* END "ShellOverlay.less" */
