// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

#ifndef ROOT_ROOT_Math_Vector2D
#define ROOT_ROOT_Math_Vector2D

// defines typedefs to specific vectors and forward declarations
#include "Math/Vector2Dfwd.h"

// coordinate system types
#include "Math/GenVector/Cartesian2D.h"
#include "Math/GenVector/Polar2D.h"

// generic Vector2D class definition
#include "Math/GenVector/DisplacementVector2D.h"

#endif
