sap.ui.define([
   'sap/ui/core/UIComponent'
], function(UIComponent) {
   "use strict";

   return UIComponent.extend("rootui5.browser.Component", {
      metadata : {
         manifest: "json"
      }
   });

});
