// @(#)root/ged:$Id: LinkDef.h,v 1.0 2003/11/25

/*************************************************************************
 * Copyright (C) 1995-2003, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TGedFrame;
#pragma link C++ class TGedNameFrame;
#pragma link C++ class TArrowEditor;
#pragma link C++ class TLineEditor;
#pragma link C++ class TAttFillEditor;
#pragma link C++ class TAttLineEditor;
#pragma link C++ class TAttMarkerEditor;
#pragma link C++ class TAttTextEditor;
#pragma link C++ class TAxisEditor;
#pragma link C++ class TFrameEditor;
#pragma link C++ class TGraphEditor;
#pragma link C++ class TTextEditor;
#pragma link C++ class TPieEditor;
#pragma link C++ class TPieSliceEditor;
#pragma link C++ class TH1Editor;
#pragma link C++ class TH2Editor;
#pragma link C++ class TF1Editor;
#pragma link C++ class TFunctionParametersDialog;
#pragma link C++ class TPaveStatsEditor;
#pragma link C++ class TPadEditor;
#pragma link C++ class TGedEditor;
#pragma link C++ class TCurlyArcEditor;
#pragma link C++ class TCurlyLineEditor;
#pragma link C++ class TGedPopup;
#pragma link C++ class TGedSelect;
#pragma link C++ class TGedPatternFrame;
#pragma link C++ class TGedPatternSelector;
#pragma link C++ class TGedPatternPopup;
#pragma link C++ class TGedPatternSelect;
#pragma link C++ class TGedMarkerSelect;
#pragma link C++ class TGedMarkerPopup;
#pragma link C++ class TStyleManager;
#pragma link C++ class TStylePreview;
#pragma link C++ class TStyleDialog;

#endif
