// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

#ifndef ROOT_ROOT_Math_Vector3D
#define ROOT_ROOT_Math_Vector3D

// defines typedefs to specific vectors and forward declarations
#include "Math/Vector3Dfwd.h"

// coordinate system types
#include "Math/GenVector/Cartesian3D.h"
#include "Math/GenVector/CylindricalEta3D.h"
#include "Math/GenVector/Polar3D.h"
#include "Math/GenVector/Cylindrical3D.h"

// generic Vector3D class definition
#include "Math/GenVector/DisplacementVector3D.h"

#endif
