/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.less";
@import "../../../../../../Base/baseLib/sap_fiori_3/base.less"; /* ORIGINAL IMPORT PATH: "../../../../../sap/ui/core/themes/sap_fiori_3/base.less" */

@import "../../../../../sap/ui/core/themes/sap_fiori_3/global.less";

@_sap_ui_table_DeleteIcon: decline;

/* START "Cell.less" */
/* ========================= */
/* CSS for sap.ui.table/Cell */
/* Fiori 3 theme             */
/* ========================= */

@_sap_ui_table_Cell_BorderColorSelected: var(--sapList_SelectionBorderColor);
@_sap_ui_table_Cell_BorderColorFixedSelected: var(--sapList_SelectionBorderColor);
@_sap_ui_table_Cell_ShadowColorFixed: @sapUiListTableFixedBorder;
@_sap_ui_table_Cell_BorderBottomFocus: transparent;
/* END "Cell.less" */

/* START "RowSelection.less" */
/* ================================== */
/* CSS for sap.ui.table/RowSelection  */
/* Fiori 3 theme                      */
/* ================================== */

@_sap_ui_table_RowSelection_CheckBoxBorderWidth: var(--sapField_BorderWidth);
@_sap_ui_table_RowSelection_CheckBoxBorderRadius: var(--sapField_BorderCornerRadius);
@_sap_ui_table_RowSelection_CheckBoxBorderRadiusCozy: var(--sapField_BorderCornerRadius);
/* END "RowSelection.less" */

/* START "Table.less" */
/* =================================== */
/* CSS for control sap.ui.table/Table  */
/* Fiori 3 theme                       */
/* =================================== */

@_sap_ui_table_Table_HeaderPaddingRight: 0.5rem;
@_sap_ui_table_Table_FooterPaddingRight: 0.5rem;
/* END "Table.less" */
