/* NOTE: This file was generated as an optimized version of "library.source.less" for the Theme Designer. */

/*!
 * OpenUI5
 * (c) Copyright 2009-2025 SAP SE or an SAP affiliate company.
 * Licensed under the Apache License, Version 2.0 - see LICENSE.txt.
 */

@import "../base/library.less";
@import "../../../../../Base/baseLib/sap_horizon_dark/base.less"; /* ORIGINAL IMPORT PATH: "../../../../sap/ui/core/themes/sap_horizon_dark/base.less" */

@import "../../../../sap/ui/core/themes/sap_horizon_dark/global.less";

/* START "InfoLabel.less" */
/* ================================== */
/* CSS for control sap.tnt/InfoLabel  */
/* Evening Horizon theme              */
/* ================================== */

// ==========================================================================
// Variables and mixins
// ==========================================================================

@_sap_tnt_InfoLabel_LineHeightDisplayOnly: 1rem;
@_sap_tnt_InfoLabel_DisplayModeBorder: none;
@_sap_tnt_InfoLabel_DisplayModeBackground: #29394B;
@_sap_tnt_InfoLabel_DisplayModeColor: #B3BDC6;
@_sap_tnt_InfoLabel_BorderRadius: 0.25rem;
@_sap_tnt_InfoLabel_DisplayBorderRadius: 0.1875rem;
@_sap_tnt_InfoLabel_Height: 1.375rem;
@_sap_tnt_InfoLabel_HeightWithoutBorders: 1.375rem;
@_sap_tnt_InfoLabel_LineHeight: 1.375rem;
@_sap_tnt_InfoLabel_Text: none;
@_sap_tnt_InfoLabel_Padding: 0 0.375rem;
@_sap_tnt_InfoLabel_DisplayPadding: 0 0.25rem;
@_sap_tnt_InfoLabel_IconMargin: 0.25rem;
@_sap_tnt_InfoLabel_DisplayIconMargin: 0.125rem;
@_sap_tnt_InfoLabel_FontFamily: @sapUiFontSemiboldDuplexFamily;
@_sap_tnt_InfoLabel_FontSize: @sapMFontMediumSize;
@_sap_tnt_InfoLabel_DisplayFontSize: @sapMFontSmallSize;
@_sap_tnt_InfoLabel_DisplayFontSizeIcon: 0.625rem;
@_sap_tnt_InfoLabel_NarrowPadding: @_sap_tnt_InfoLabel_Padding;
@_sap_tnt_InfoLabel_LetterSpacing: normal;

@_sap_tnt_InfoLabel_ColorsArray:
@sapUiAvatar1Background @sapUiAvatar1TextColor,
@sapUiAvatar2Background @sapUiAvatar2TextColor,
@sapUiAvatar3Background @sapUiAvatar3TextColor,
@sapUiAvatar4Background @sapUiAvatar4TextColor,
@sapUiAvatar5Background @sapUiAvatar5TextColor,
@sapUiAvatar6Background @sapUiAvatar6TextColor,
@sapUiAvatar7Background @sapUiAvatar7TextColor,
@sapUiAvatar8Background @sapUiAvatar8TextColor,
@sapUiAvatar9Background @sapUiAvatar9TextColor,
@sapUiAvatar10Background @sapUiAvatar10TextColor;

.create-classes-for-background-and-border-colors(backgroundColor, @_sap_tnt_InfoLabel_ColorsArray);

// Iterate over the InfoLabel ColorsArray and create separate class for each
// color named with the passed prefix and the number of the color
.create-classes-for-background-and-border-colors(@prefix, @list) {
	.iter(length(@list));
	.iter(@i) when (@i > 0) {
		.iter(@i - 1);

		@pair: extract(@list, @i);
		@backgroundColor: extract(@pair, 1);
		@textColor: extract(@pair, 2);

		.@{prefix}@{i} {
			background-color: @backgroundColor;

			.sapTntInfoLabelInner {
				color: @textColor;
			}
		}
	}
}
/* END "InfoLabel.less" */

/* START "SideNavigation.less" */
/* ======================================= */
/* CSS for control sap.tnt/SideNavigation  */
/* Evening Horizon theme                   */
/* ======================================= */

@_sap_tnt_SideNavigation_Width: 16rem;
@_sap_tnt_SideNavigation_CollapsedWidth: 4rem;
@_sap_tnt_SideNavigation_NavigationSeparatorMargin: 0.25rem 0.5rem 0.25rem 0.5rem;
@_sap_tnt_SideNavigation_NavigationSeparatorBackgroundColor: @sapUiToolbarSeparatorColor;
@_sap_tnt_SideNavigation_NavigationSeparatorRadius: 0.125rem;
@_sap_tnt_SideNavigation_NavigationSeparatorHeight: 0.0625rem;
@_sap_tnt_SideNavigation_TriangleColor: @sapUiContentNonInteractiveIconColor;
@_sap_tnt_SideNavigation_PhoneBorderRadius: 0.5rem;
@_sap_tnt_SideNavigation_BoxShadow: var(--sapContent_Shadow0);
@_sap_tnt_SideNavigation_SeparatorMargin: 0.3125rem 0.5rem 0.3125rem 0.5rem;
@_sap_tnt_SideNavigation_TriangleDisplay: none;
@_sap_tnt_SideNavigation_PhoneWidth: ~"100% !important";
/* END "SideNavigation.less" */

/* START "NavigationList.less" */
/* ======================================= */
/* CSS for control sap.tnt/NavigationList  */
/* Evening Horizon theme                   */
/* ======================================= */

@_sap_tnt_NavigationList_CollapsedWidth: 4rem;
@_sap_tnt_NavigationList_IconColor: @sapUiListTextColor;
@_sap_tnt_NavigationList_ExpandIconColor: @sapUiListTextColor;
@_sap_tnt_NavigationList_GroupBorderStyleColor: none;
@_sap_tnt_NavigationList_ItemHeight: 2.75rem;
@_sap_tnt_NavigationList_ItemBorderRadius: 0.375rem;
@_sap_tnt_NavigationList_ItemBottomMargin: 0.25rem;
@_sap_tnt_NavigationList_ItemBottomMarginInOverflowPopup: 0.25rem;
@_sap_tnt_NavigationList_ItemFocusBorderOffset: -@sapUiContentFocusWidth;
@_sap_tnt_NavigationList_ItemFocusBorderRadius: @_sap_tnt_NavigationList_ItemBorderRadius + @sapUiContentFocusWidth;
@_sap_tnt_NavigationList_ItemTransition: background-color 0.3s ease-in-out, max-width 0.3s ease-in-out;
@_sap_tnt_NavigationList_SelectedBorderStyleColor: none;
@_sap_tnt_NavigationList_Collapsed_SelectedItemBorderStyleColor: solid @_sap_tnt_NavigationList_SelectedItemBorderColor;
@_sap_tnt_NavigationList_SelectedItemBorder: none;
@_sap_tnt_NavigationList_Collapsed_SelectedItemBorder: 0.125rem solid @_sap_tnt_NavigationList_SelectedItemBorderColor;
@_sap_tnt_NavigationList_GroupIconWidth: 2rem;
@_sap_tnt_NavigationList_GroupIconWidthExpanded: 1rem;
@_sap_tnt_NavigationList_GroupIconMarginLeft: 1rem;
@_sap_tnt_NavigationList_GroupIconMarginRight: 0.5rem;
@_sap_tnt_NavigationList_ParentItemBottomMarginInPopup: 0.5rem;
@_sap_tnt_NavigationList_GroupTextWeight: bold;
@_sap_tnt_NavigationList_GroupPadding: 0;
@_sap_tnt_NavigationList_NoIconsNestedItemPadding: 2rem;
@_sap_tnt_NavigationList_Padding: 0.5rem 0.5rem 0 0.5rem;
@_sap_tnt_NavigationList_PaddingFixed: 0 0.5rem 0.5rem 0.5rem;
@_sap_tnt_NavigationList_PopupPadding: 0.5rem;
@_sap_tnt_NavigationList_PopupItemPadding: 0 0.5rem;
@_sap_tnt_NavigationList_PopupTitleTextSize: var(--sapFontSize);
@_sap_tnt_NavigationList_PopupTitleLineHeight: 1.5rem;
@_sap_tnt_NavigationList_Collapsed_HoverOrFocusTextDisplay: block;
@_sap_tnt_NavigationList_Collapsed_HoverOrFocusItemZIndex: 1;
@_sap_tnt_NavigationList_Collapsed_HoverOrFocusItemPosition: absolute;
@_sap_tnt_NavigationList_Collapsed_HoverOrFocusItemPadding: 0 0 0 0.5rem;
@_sap_tnt_NavigationList_Collapsed_HoverOrFocusItemPaddingCompact: @_sap_tnt_NavigationList_Collapsed_HoverOrFocusItemPadding;
@_sap_tnt_NavigationList_CollapsedIconPaddingLeft: 0.25rem;
@_sap_tnt_NavigationList_NavigationIndicator: "\E066";
@_sap_tnt_NavigationList_ExpandIconWidth: 2.25rem;
@_sap_tnt_NavigationList_ExpandIconMarginRight: 0.375rem;
@_sap_tnt_NavigationList_ExpandIconMarginLeft: 0.25rem;
@_sap_tnt_NavigationList_ExpandIconPaddingLeft: 0.25rem;
@_sap_tnt_NavigationList_ExpandIconFontSize: 1rem;
@_sap_tnt_NavigationList_Collapsed_SelectedItemBackground: ~"0 100% / 0.188rem 100%" no-repeat linear-gradient(0deg, @sapUiListHighlightColor, @sapUiListHighlightColor), @sapUiListSelectionBackgroundColor;
@_sap_tnt_NavigationList_Collapsed_SelectedItemBackgroundHover: @sapUiListSelectionHoverBackground;
@_sap_tnt_NavigationList_Collapsed_SelectedItemBackgroundRTL: ~"100% 0 / 0.188rem 100%" no-repeat linear-gradient(0deg, @sapUiListHighlightColor, @sapUiListHighlightColor), @sapUiListSelectionBackgroundColor;
@_sap_tnt_NavigationList_ActionItemBorderHover: @_sap_tnt_NavigationList_ActionItemBorder;
@_sap_tnt_NavigationList_ActionItemBorderPosition: -1px;
@_sap_tnt_NavigationList_ActionItemBorderHeight: auto;
@_sap_tnt_NavigationList_ActionItemBorderPaddingLeft: calc(@_sap_tnt_NavigationList_Collapsed_HoverOrFocusItemPadding ~"-" 0.0625rem);

/* END "NavigationList.less" */
 // keep this order, it reuses variable @_sap_tnt_SideNavigation_BoxShadow defined in SideNavigation.less
/* START "ToolHeader.less" */
/* =================================== */
/* CSS for control sap.tnt/ToolHeader  */
/* Evening Horizon theme               */
/* =================================== */

@_sap_tnt_ToolHeader_Height: 3.25rem;
@_sap_tnt_ToolHeader_IconHeight: 0.75rem;
@_sap_tnt_ToolHeader_DisabledOpacity: @sapUiContentDisabledOpacity;
@_sap_tnt_ToolHeader_ArrowHeight: 0.125rem;
@_sap_tnt_ToolHeader_ArrowMargin: 0 -0.25rem;
@_sap_tnt_ToolHeader_Padding: 0 0.75rem;
@_sap_tnt_ToolHeader_ArrowMarginBottom: 0.25rem;
@_sap_tnt_ToolHeader_IconTabHeader_Height: auto;

// ==========================================================================
// sap.m.Toolbar overriding
// ==========================================================================

.sapTntToolHeader:not(.sapUshellShellToolHeader) {
	&.sapMTB {
		background-color: @sapUiShellColor;
		padding: 0 1rem 0 0.875rem;
	}

	&.sapMTBStandard {
		border-bottom: none;
	}
}

.sapTntToolHeader {
	&.sapMIBar.sapMTB .sapMBarChild.sapMTBSeparator {
		background: none;
		width: 0.75rem;
	}
}

// ==========================================================================
// sap.m.Button overriding
// ==========================================================================

.sapTntToolHeader {
	.sapMBarChild:not(.sapMMenuBtn) {
		&.sapMBtn {
			.sapMBtnInner::after,
			.sapMBtnIcon {
				background: transparent;
				color: @sapUiShellInteractiveTextColor;
			}
			.sapMBtnContent {
				background: transparent;
				color: @sapUiShellTextColor;
			}

			&:not(.sapMBtnDisabled) > span.sapMBtnInner.sapMBtnActive {
				background: @sapUiShellActiveBackground;
				border-color: @sapUiShellActiveBackground;
			}
			.sapMToggleBtnPressed:not(.sapMBtnDisabled) {
				background-color: @sapUiShellHoverBackground;
				.sapMBtnIcon {
					color: @sapUiShellActiveTextColor;
				}
			}
			&:hover:not(.sapMBtnDisabled) {
				.sapMToggleBtnPressed:not(.sapMBtnActive), .sapMToggleBtnPressed:not(.sapMBtnDisabled):hover  {
					background-color: @sapUiShellHoverBackground;
					.sapMBtnIcon {
						color: @sapUiShellActiveTextColor;
					}
				}
				>span.sapMBtnInner.sapMBtnHoverable:not(.sapMToggleBtnPressed) {
					&:not(.sapMBtnActive) {
						background: @sapUiShellHoverBackground;
						border: 1px solid @sapUiButtonLiteHoverBorderColor;
					}
					&.sapMBtnActive {
						background: @sapUiShellActiveBackground;
						color: @sapUiShellActiveTextColor;
						border: 1px solid @sapUiButtonLiteActiveBorderColor;

							.sapMBtnContent bdi {
								color: @sapUiShellActiveTextColor;
							}
					}
					&.sapMBtnActive:after,
					&.sapMBtnActive >.sapMBtnIcon {
						color: @sapUiShellActiveTextColor;
					}
					&:after,
					>.sapMBtnIcon {
						color: @sapUiShellInteractiveTextColor;
					}
					>.sapMBtnContent,
					>.sapMBtnContent bdi
					{
						color: @sapUiShellTextColor;
					}
				}

				&:active {
					>.sapMBtnContent,
					>.sapMBtnInner,
					>.sapMBtnIcon {
						color: @sapUiShellTextColor;
					}
				}
			}
		}
	}
}

.sapUiSizeCompact {
	.sapTntToolHeader {
		.sapMBarChild:not(.sapMMenuBtn) {
			&.sapMBtn {
				padding: 0;
				min-width: 2rem;

				.sapMBadgeTopRight.sapMBadge .sapMBadgeIndicator {
					position: absolute;
					top: -0.3125rem;
					right: -0.4375rem;
					padding-right: 0.125rem;
				}

				.sapMBadgeTopRight.sapMBadge.sapMBtnIconFirst:not(.sapMBtnText) {
					padding: 0;
				}
			}

			&.sapMBtn .sapMBtnTransparent,
			&.sapMBtn .sapMBtnDefault,
			&.sapMBtn .sapMBtnBack,
			&.sapMBtn .sapMBtnUp {
				height: 2rem;

				.sapUiIcon {
					line-height: 2rem;
				}

				.sapMBtnContent {
					line-height: 2rem;
				}
			}
		}

		.sapMSF {
			padding: 0 0;
		}
	}
}

// ==========================================================================
// sap.m.Text, sap.m.Title and sap.m.Label overriding
// ==========================================================================

.sapTntToolHeader {
	.sapMTitle {
		text-shadow: none;
		font-family: @sapFontSemiboldFamily;
		font-size: @sapFontHeader5Size;
		line-height: 1.625rem;
		color: @sapUiShellSubBrandTextColor;
	}

	.sapMText,
	.sapMLabel {
		line-height: @sapUiContentLineHeight;
		color: @sapUiContentLabelColor;
	}

	&.sapMIBar.sapMTB .sapMBarChild.sapMText,
	&.sapMIBar.sapMTB .sapMBarChild.sapMLabel {
		margin-top: 2px; // to align the text/label with the title
		margin-left: 0.5rem;
	}
}

.sapUiSizeCompact {
	.sapTntToolHeader {
		&.sapMIBar.sapMTB .sapMBarChild.sapMText,
		&.sapMIBar.sapMTB .sapMBarChild.sapMLabel {
			margin-top: 3px; // to align the text/label with the title
		}
	}
}

// ==========================================================================
// sap.m.SearchField overriding
// ==========================================================================

.sapTntToolHeader {

	.sapMSFF {
		background: @sapUiFieldBackgroundStyle;
		background-color: @sapUiShellInteractiveBackground;
		box-shadow: @sapUiFieldShadow;
		border: none;
		color: @sapUiShellInteractiveTextColor;
		border-radius: 1.125rem;
		height: 2.25rem;

		.sapMSFB {
			width: 1.75rem;
			height: 1.75rem;
			line-height: 1.75rem;
			font-size: 1rem;
			text-align: center;
			margin-top: 0.25rem;
			padding: 0;
			right: 0;
			top: 0;
			margin-right: 0.25rem;
		}

		.sapMSFB:hover {
			background: @sapUiShellHoverBackground;
			box-shadow: @sapUiButtonLiteHoverBorderColor;
			border-radius: 1.125rem;
			height: 1.75rem;
			color: @sapUiShellInteractiveTextColor;
		}

		.sapMSFR {
			margin-right: 0;
		}

		.sapMSFB:active,
		.sapMSFB.sapMSFBA {
			background: @sapUiShellActiveBackground;
			box-shadow: @sapUiContentSelectedShadow;
			border-radius: 1.125rem;
			color: @sapUiShellActiveTextColor;
		}
	}
}

.sapUiSizeCompact {
	.sapTntToolHeader {
		.sapMSF {
			height: 2rem;
		}

		.sapMSF .sapMSFF {
			height: 2rem;
		}

		.sapMSFB {
			margin-top: 0.125rem;
			width: 1.625rem;
			height: 1.625rem;
		}

		.sapMSFS {
			margin-right: 0.1875rem;
		}
	}
}

// ==========================================================================
// sap.m.IconTabHeader overriding
// ==========================================================================

.sapTntToolHeader {
	.sapUshellShellTabBar .sapMITH,
	.sapUshellShellTabBar.sapMITH {
		box-shadow: none;
	}
}

// ==========================================================================
// sap.m.Image overriding
// ==========================================================================

.sapTntToolHeader {
	.sapMImg {
		height: 1.625rem;
	}

	&.sapMIBar.sapMTB {
		.sapMBarChild.sapMImg:first-child,
		.sapMBarChild.sapMBtn + .sapMBarChild.sapMImg:first-of-type {
			margin-left: 0.5rem;
		}
	}

	.sapMImgFocusable:focus {
		outline: @sapUiContentFocusStyle @sapUiContentFocusWidth @sapUiContentFocusColor;
		outline-offset: 0.125rem;
		border-radius: 0.25rem;
	}
}
/* END "ToolHeader.less" */

/* START "ToolPage.less" */
/* ================================= */
/* CSS for control sap.tnt/ToolPage  */
/* Evening Horizon theme             */
/* ================================= */

@_sap_tnt_ToolPage_ContentBackgroundStandard: @sapUiBaseBG;
@_sap_tnt_ToolPage_ContentBackgroundSolid: @sapUiBaseBG;
@_sap_tnt_ToolPage_ContentBackgroundTransparent: transparent;
@_sap_tnt_ToolPage_ContentBackgroundList: @sapUiGroupContentBackground;
@_sap_tnt_ToolPage_AsideCollapsedPhoneTransform: calc(~"-100% - 0.25rem");
@_sap_tnt_ToolPage_AsideCollapsedPhoneTransformRtl: calc(~"100% + 0.25rem");
@_sap_tnt_ToolPage_AsidePhoneTop: -0.4375rem;
@_sap_tnt_ToolPage_AsidePhoneWidth: 100%;
@_sap_tnt_ToolHeader_Shadow: @sapUiShellShadow;
/* END "ToolPage.less" */
