// Standard import from ROOT build
export * from '../rcore/REveRenderCore-min.mjs';
export const REveShaderPath = "rcore/shaders/";
export const REveDevelMode = false;

// Development import from a RenderCore checkout in RC directory
// export * from '../RC/src/contrib/REveRenderCore.js';
// export const REveShaderPath = "RC/src/shaders/";
// export const REveDevelMode = true;