/* @(#)root/gpad:$Id$ */

/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TAnnotation+;
#pragma link C++ class TAttCanvas+;
#pragma link C++ class TButton+;
#pragma link C++ class TCanvas-;
#pragma link C++ class TClassTree-;
#pragma link C++ class TColorWheel+;
#pragma link C++ class TControlBar+;
#pragma link C++ class TControlBarButton+;
#pragma link C++ class TDialogCanvas+;
#pragma link C++ class TGroupButton+;
#pragma link C++ class TInspectCanvas+;
#pragma link C++ class TPad-;
#pragma link C++ class TPaveClass+;
#pragma link C++ class TSlider+;
#pragma link C++ class TSliderBox+;
#pragma link C++ class TView+;
#pragma link C++ class TViewer3DPad;
#pragma link C++ class TPadPainterBase;
#pragma link C++ class TPadPainter;
#pragma link C++ class TPadPainterPS;
#pragma link C++ class TRatioPlot+;

#endif
