// @(#)$Id$
/*************************************************************************
 * Copyright (C) 1995-2000, Rene Brun and Fons Rademakers.               *
 * All rights reserved.                                                  *
 *                                                                       *
 * For the licensing terms see $ROOTSYS/LICENSE.                         *
 * For the list of contributors see $ROOTSYS/README/CREDITS.             *
 *************************************************************************/

#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class TGeoVolumeEditor;
#pragma link C++ class TGeoBBoxEditor;
#pragma link C++ class TGeoTubeEditor;
#pragma link C++ class TGeoTubeSegEditor;
#pragma link C++ class TGeoCtubEditor;
#pragma link C++ class TGeoEltuEditor;
#pragma link C++ class TGeoHypeEditor;
#pragma link C++ class TGeoConeEditor;
#pragma link C++ class TGeoConeSegEditor;
#pragma link C++ class TGeoTrd1Editor;
#pragma link C++ class TGeoParaEditor;
#pragma link C++ class TGeoTorusEditor;
#pragma link C++ class TGeoPconEditor;
#pragma link C++ class TGeoPconSection;
#pragma link C++ class TGeoPgonEditor;
#pragma link C++ class TGeoTrd2Editor;
#pragma link C++ class TGeoTrapEditor;
#pragma link C++ class TGeoGtraEditor;
#pragma link C++ class TGeoSphereEditor;
#pragma link C++ class TGeoMediumEditor;
#pragma link C++ class TGeoMaterialEditor;
#pragma link C++ class TGeoMixtureEditor;
#pragma link C++ class TGeoNodeEditor;
#pragma link C++ class TGeoTranslationEditor;
#pragma link C++ class TGeoRotationEditor;
#pragma link C++ class TGeoCombiTransEditor;
#pragma link C++ class TGeoManagerEditor;
#pragma link C++ class TGeoTabManager;
#pragma link C++ class TGeoTransientPanel;
#pragma link C++ class TGeoTreeDialog;
#pragma link C++ class TGeoVolumeDialog;
#pragma link C++ class TGeoShapeDialog;
#pragma link C++ class TGeoMatrixDialog;
#pragma link C++ class TGeoMediumDialog;
#pragma link C++ class TGeoMaterialDialog;
#pragma link C++ class TGeoGedFrame;

#endif
