// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

#ifndef ROOT_Math_GenVector_AxisAnglefwd
#define ROOT_Math_GenVector_AxisAnglefwd  1


namespace ROOT {

  namespace Math {

  /**
      Class describing a rotation represented
      by an axis and an angle of rotation
   */

class AxisAngle;

}  // namespace Math
}  // namespace ROOT

#endif  // ROOT_Math_GenVector_AxisAnglefwd
