#ifdef __CLING__

#pragma link off all globals;
#pragma link off all classes;
#pragma link off all functions;

#pragma link C++ class THbookFile+;
#pragma link C++ class THbookTree+;
#pragma link C++ class THbookBranch+;
#pragma link C++ class THbookKey+;

#endif
