// @(#)root/mathcore:$Id$
// Authors: W. Brown, M. Fischler, L. Moneta    2005

#ifndef ROOT_Math_GenVector_DisplacementVector3Dfwd
#define ROOT_Math_GenVector_DisplacementVector3Dfwd  1


namespace ROOT {
  namespace Math {

    /**
        Class template describing a 3D displacement vector
     */
    template< class CoordSystem, class Tag  >
    class DisplacementVector3D;

  }  // namespace Math
}  // namespace ROOT


#endif
